<?php
namespace App\Http\Controllers;

use App\Http\ResponseHelper;
use App\Model\Shopify\ExtentedShopifyAPI;
use Config;
use DB;
use Illuminate\Http\Request;
use Log;
use App\Http\Controllers\ShopifyMultiPass;

class CommonController extends Controller
{
    private $helper;
    private $shopify_api_version;

    public function __construct()
    {
        $this->helper = new ResponseHelper();
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    /**
     * Function for create a customer in shopify
     * This function not for a quiz user
     * params : email, password
     */
    public function createShopifyCustomer(Request $request)
    {
        $api = new ExtentedShopifyAPI();

        $password = Config::get('shopify.passwordQuiz');
        $shopDomain = Config::get('shopify.shopDomainQuiz');

        $api->setShop($shopDomain);
        $api->setAccessToken($password);

        Log::info($request->input());

        $email = $request->input('email');
        $custPassword = $request->input('password');

        if (empty($email)) {
            $returnData = [
                'error' => 'Parameter missing - email',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        if (empty($custPassword)) {
            $returnData = [
                'error' => 'Parameter missing - password',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        if (strlen($custPassword) < 5) {
            $returnData = [
                'error' => 'Password is too short (minimum is 5 characters)',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        $userEmailDetails = DB::table('users')
            ->select('*')
            ->where('users.user_email', '=', $email)
            ->get();

        if (isset($userEmailDetails[0]->user_email)) {

            $returnData = [
                'error' => 'Email has already been taken',
            ];

            return $this->helper->returnData(600, "error", $returnData);
        } else {

            $tags = [];
            $tags[] = "passwordSet";

            $params = array(
                'customer' => array(
                    'email' => $email,
                    "send_email_welcome" => "false",
                    "password" => $custPassword,
                    "password_confirmation" => $custPassword,
                    "tags" => $tags,
                    "accepts_marketing" => "true"
                ),
            );

            Log::info('---------customer creation endpoint---------');
            Log::info(json_encode($params));

            $retry = false;
            $try = 1;
            $result = '';
            do {
                Log::info($try . ' try-create shopify customer ');
                $result = $api->rest('POST', '/admin/api/' . $this->shopify_api_version . '/customers.json', $params);
                if (!isset($result->body->customer->id)) {
                    $retry = true;
                    $try++;
                }

            } while ($retry && $try <= 3);
            Log::info(json_encode($result));

            if (isset($result->body->customer->id)) {
                
                /* code changes for multipass login */
                $customer_login_data = array(
                    "email" => $email
                );
                $multiPassToken = env('MULTI_PASS_TOKEN', true);
                $multipass = new ShopifyMultipass($multiPassToken);
                $token = $multipass->generate_token($customer_login_data);
    
                /* code changes for multipass login */
                
                $returnData = [
                    'success' => 'Customer successfully created.',
                    'multi_pass_token' => ($token != '')?$token:"",
                ];

                return $this->helper->returnData(200, "success", $returnData);
            } else {

                // echo(json_encode($result));die;
                if (isset($result->errors->body->errors)) {
                    $returnData = [
                        'error' => 'Email ' . $result->errors->body->errors->email[0],
                    ];

                } else {
                    $returnData = [
                        'error' => "Invalid email or password",
                    ];
                }
                return $this->helper->returnData(600, "error", $returnData);

            }
        }
    }

}
