<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use Log;
use Illuminate\Http\Request;
use App\Model\Shopify\ExtentedShopifyAPI;
use OhMyBrew\BasicShopifyAPI;
use Config;
use App\Model\RechargeApi;

class CustomerController extends Controller {

    private $helper;
    private $shopify_api_version;

    function __construct() {

        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    public function getCustomerActivationStatus(Request $request) {

        // $api = new BasicShopifyAPI();
        $api = new ExtentedShopifyAPI();
        $password   = Config::get('shopify.passwordCustomer');
        $shopDomain = Config::get('shopify.shopDomainCustomer');

        $api->setShop($shopDomain);
        $api->setAccessToken($password);

        //Log::info($_REQUEST['email']);
        
        $email = $request->input('email');
        if(empty($email)) {
            $returnData = [
                'error' => 'Parameter missing - email'
            ];

             $this->sendResponse(600, "error", $returnData );
        }
       
        $params = NULL;
        $result = $api->rest('GET', 
                htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/search.json?query=email:'.$email), $params); 

        

        if(!isset($result->body->customers[0]) || empty($result->body->customers[0])) {
            
            $returnData = [
                "customer_available" => false,
                "activation_status" => null
        
            ];

            $this->sendResponse(200, "success", $returnData );
        }
        else {
            $customer = [];
            foreach($result->body->customers as $cust) {
                if($cust->email == $email) {
                    $customer = $cust;
                    break;
                }
            }
            $status = $customer->state;

            $returnData = [
                "customer_available" => true,
                "activation_status" => $status
        
            ];

            $this->sendResponse(200, "success", $returnData );
        }

    }

    public function sendResponse($status,$message, $body) {
        $response = array(
            "status"       => $status,
            "message" => $message,
            "body" => $body
        );
        echo json_encode($response);
        die();

    }

    public function getActivationUrl(Request $request) {
 
        // $api = new BasicShopifyAPI();
        $api = new ExtentedShopifyAPI();
        $password   = Config::get('shopify.passwordCustomer');
        $shopDomain = Config::get('shopify.shopDomainCustomer');

        $api->setShop($shopDomain);
        $api->setAccessToken($password);

        //Log::info($_REQUEST['email']);
       
        $email = $request->input('email');
        
        if (empty($email)) {
            $returnData = [
                'error' => 'Parameter missing - email'
            ];

             $this->sendResponse(600, "error", $returnData );
        }
       
        $params = NULL;
        $result = $api->rest('GET', 
                htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/search.json?query=email:'.$email), $params); 
                
        if (!isset($result->body->customers[0]) || empty($result->body->customers[0])) {
            
            $returnData = [
                'error' => 'email not found'
            ];

            $this->sendResponse(600, "error", $returnData );
        }
        $customer = '';
        foreach($result->body->customers as $cust) {
            if(strtolower(trim($cust->email)) == strtolower(trim($email))) {
                $customer = $cust;
                break;
            }
        }

        if (empty($customer)) {

           $returnData = [
                'error' => 'email not found'
            ];

            $this->sendResponse(600, "error", $returnData );
        }

        if (isset($customer->state) && strtolower(trim($customer->state)) == "enabled") {
            $returnData = [
                "activation_status" => "enabled"
            ];

             $this->sendResponse(200, "success", $returnData );
        }

        
        $result = $api->rest('POST', 
                htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/'.$customer->id.'/account_activation_url.json'), $params);
        
        
        if (!isset($result->body->account_activation_url) || empty($result->body->account_activation_url)) {

            $returnData = [
                'error' => 'Failed to create activation URL'
            ];
             $this->sendResponse(600, "error", $returnData );
        }
        else {
            // $returnData = [
            //     "activation_status" => "disabled",
            //     "activation_url" => $result->body->account_activation_url
            // ];
            
            $activationURL = $result->body->account_activation_url;
            
            $iterableMailRequest = array(

                "campaignId" => intval(env('ITERABLE_CUSTOMER_ACTIVATION_CAMPAIGN_ID',true)),
                "recipientEmail" => $email,
                "dataFields" => array(
                    "customUrl" => $activationURL,
                )

            );

            $url = env('ITERABLE_URL',true).'email/target?api_key='.env('ITERABLE_API_KEY',true);
            $method = 'POST';

            $triggerResponse = $this->executeCurl($url,$method,json_encode($iterableMailRequest));

             $returnData = [
                 "activation_status" => "disabled",
                 "mail_status" => "sent"
             ];

            $this->sendResponse(200, "success", $returnData );
        }

    }
    
    public function returnResponse($status,$message, $body,$request) {
        $response = array(
            "status"       => $status,
            "message" => $message,
            "body" => $body
        );
        echo $request['callback'] . '(' . json_encode($response) . ')';
        die();

    }
    
    public function getRechargeHash() {
        
        
        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $returnData = [
                'message' => $message
            ];
            $this->returnResponse(600, "error", $returnData,$_REQUEST );
        }
        
        $receivedData = $_REQUEST['data'];
        $receivedData = json_decode($receivedData);
        
        // print_r($receivedData);die;
        
        if (!isset($receivedData->email) || empty($receivedData->email)) {
            
            $returnData = [
                'message' => 'Parameter missing - email'
            ];

            $this->returnResponse(600, "error", $returnData,$_REQUEST );

        }

         if (!isset($receivedData->customerId) || empty($receivedData->customerId)) {
            
            $returnData = [
                'message' => 'Parameter missing - customerId'
            ];

            $this->returnResponse(600, "error", $returnData,$_REQUEST );

        }

        // $api = new BasicShopifyAPI();
        $shopifyApiObj = new ExtentedShopifyAPI();
        $rechargeApiObj = new RechargeApi();
        $password   = Config::get('shopify.passwordCustomer');
        $shopDomain = Config::get('shopify.shopDomainCustomer');

        $shopifyApiObj->setShop($shopDomain);
        $shopifyApiObj->setAccessToken($password);

        $rechargeHash = null;
        
        
        //Log::info($_REQUEST['email']);
        
        $email = trim($receivedData->email);
        $customerId = trim($receivedData->customerId);
        
       
        $params = NULL;
        $result = $shopifyApiObj->rest('GET', 
                htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json'), $params); 

        

        if(!isset($result->body->customer) || empty($result->body->customer)) {
            
            $returnData = [                
                "message" => "Invalid Customer details."            
            ]; 

            $this->returnResponse(600, "error", $returnData, $_REQUEST );
        }
        else {
            $customer = $result->body->customer;

            if( trim($customer->email) == $email && trim($customer->id) == $customerId ){

                $rechargeResults = $rechargeApiObj->getRechargeCustomerDetailsByEmail($email);
                
                

                if(isset($rechargeResults->error) || isset($rechargeResults->errors)) {
            
                    $returnData = [                
                        "message" => "Customer details not available in recharge."            
                    ];         
                    $this->returnResponse(600, "error", $returnData,$_REQUEST );
                }else{
                    // print_r($rechargeResults->customers);die;
                    foreach($rechargeResults->customers as $rechargeCustomer) {
                        if($rechargeCustomer->email == $email) {
                            $rechargeHash = $rechargeCustomer->hash;
                            break;
                        }
                    }
                }
            }

            if($rechargeHash != null){
                $returnData = [
                    "customerEmail" => $email,
                    "rechargeHash"  => $rechargeHash            
                ];
                $this->returnResponse(200, "success", $returnData,$_REQUEST );
            }else{
                $returnData = [                
                    "message" => "Invalid customer details"            
                ];         
                $this->returnResponse(600, "error", $returnData,$_REQUEST );
            }            

            
        }

    }
    
    public function getCustomerStanding() {
        
        
        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $returnData = [
                'message' => $message
            ];
            $this->returnResponse(600, "error", $returnData,$_REQUEST );
        }
        
        $receivedData = $_REQUEST['data'];
        $receivedData = json_decode($receivedData);
        
        // print_r($receivedData);die;
        
        if (!isset($receivedData->email) || empty($receivedData->email)) {
            
            $returnData = [
                'message' => 'Parameter missing - email'
            ];

            $this->returnResponse(600, "error", $returnData,$_REQUEST );

        }

         if (!isset($receivedData->orderId) || empty($receivedData->orderId)) {
            
            $returnData = [
                'message' => 'Parameter missing - orderId'
            ];

            $this->returnResponse(600, "error", $returnData,$_REQUEST );

        }

        // $api = new BasicShopifyAPI();
        $shopifyApiObj = new ExtentedShopifyAPI();       
        $password   = Config::get('shopify.passwordCustomer');
        $shopDomain = Config::get('shopify.shopDomainCustomer');

        $shopifyApiObj->setShop($shopDomain);
        $shopifyApiObj->setAccessToken($password);          
       
        
        $requestEmail = trim($receivedData->email);
        $currentOrderId = trim($receivedData->orderId);
        $customerAvailable = false;
        

        $customerResult = $shopifyApiObj->rest('GET',
                    htmlentities('/admin/api/'.$this->shopify_api_version.'/customers/search.json?query=email:'.$requestEmail), null);

        if (!isset($customerResult->body->customers[0]) || empty($customerResult->body->customers[0])) {
            $returnData = [
                'message' => 'No such customer found in shopify',
                "customerStanding" => null 
            ];

            $this->returnResponse(200, "success", $returnData, $_REQUEST );
        }else{
            $customers = $customerResult->body->customers;            
            foreach( $customers as $customer){
                if($customer->email == $requestEmail){
                    $customerAvailable = true;
                }
            }
            if(!$customerAvailable){
                $returnData = [
                    'message' => 'No such customer found in shopify',
                    "customerStanding" => null 
                ];
                $this->returnResponse(200, "success", $returnData,$_REQUEST );
            }
        }
       
       
        $params = NULL;
        $result = $shopifyApiObj->rest('GET', 
                htmlentities('/admin/api/'.$this->shopify_api_version.'/orders.json?selectedView=all&status=any&query='.$requestEmail), $params); 

        
        if($customerAvailable && (!isset($result->body->orders) || empty($result->body->orders))) {
            
            $returnData = [
                "customerStanding" => 'New'                           
            ];
            $this->returnResponse(200, "success", $returnData,$_REQUEST );;
        }
        else {
            $orders = $result->body->orders;  
            $orderDetails = [];
            foreach( $orders as $order){
                if($order->email == $requestEmail){
                    $orderDetails[] = $order->id;
                }
            }
            

            if(count($orderDetails) > 1){
                $returnData = [
                    "customerStanding" => 'Returning'                           
                ];
                $this->returnResponse(200, "success", $returnData,$_REQUEST );
            }    
            
            if(count($orderDetails) == 1 && in_array($currentOrderId,$orderDetails) ){
                $returnData = [
                    "customerStanding" => 'New'                           
                ];
                $this->returnResponse(200, "success", $returnData,$_REQUEST );
            }else{
                $returnData = [
                    "customerStanding" => 'New'                           
                ];
                $this->returnResponse(200, "success", $returnData,$_REQUEST );
            }
            
        }

    }


    public function executeCurl($url,$type,$data) {
	    
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
	    curl_setopt($ch, CURLOPT_HEADER, FALSE);
	    curl_setopt($ch, CURLOPT_HTTPHEADER, array(
	            "Content-Type: application/json"
	    ));
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE); // temporarily disable SSL verification
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE); // temporarily disable SSL verification

	    if ($type == "POST"){
	        curl_setopt($ch, CURLOPT_POST, TRUE);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
	    }

	    $result = curl_exec($ch);

	    if(curl_errno($ch)){
	        echo 'Request Error:' . curl_error($ch);
	        return false;
        }
        
        Log::info("curl result  => ".$result);
	    return $result;
    }
}
