<?php

namespace App\Http\Controllers;

use DB;
use Log;
use App\Users;
use Illuminate\Http\Request;
use App\Http\ResponseHelper;
use App\Jobs\GenerateUserTokenJob;
use Config;

class CustomerCreationWebhookController extends Controller
{

    private $helper;
    private $shopifyProductDao;
    private $sharedSecret;
    private $randomString;

    public function __construct()
    {
        $this->helper = new ResponseHelper();
    }

    function processWebhook()
    {       
        $this->sharedSecret  = Config::get('shopify.secret');
        $this->webhookContent    = $this->webhook($this->sharedSecret);
        
        $this->randomString = $this->generateRandomString(16);
        $queuedJobResponse = dispatch(new GenerateUserTokenJob($this->webhookContent, $this->randomString)); 

        ignore_user_abort(true);
        header("Content-Length: 0");
        header("Connection: Close");
        flush();
        
    }

    function webhook($my_shared_secret){
        
        define('SHOPIFY_APP_SECRET', 'my_shared_secret');
        $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
        $data        = file_get_contents('php://input');
        $verified    = $this->verify_webhook($data, $hmac_header);
        error_log('Webhook verified: '.var_export($verified, true));
        return $data;

    }

    function verify_webhook($data, $hmac_header){

        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, SHOPIFY_APP_SECRET, true));
        return ($hmac_header==$calculated_hmac);

    }

    function generateRandomString($length) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }
    

    

}
