<?php

namespace App\Http\Controllers;

use App\Http\Requests\DraftOrderValidation;
use Illuminate\Http\Request;
use App\Model\Shopify\Product;
use OhMyBrew\BasicShopifyAPI;
use Config;
use stdClass;

class DraftOrderController extends Controller
{

    private $productObj;
    private $shopify_api_version;
     /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {
        $this->productObj = new Product();
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    // To  draft order view
    public function createDraftOrder($status = []) {
        $prod  = $this->setProductDetails();
        $jsonProductDetails = json_encode($prod); 
        $status = json_encode($status); 
        return view('draft_order',compact('jsonProductDetails'),compact('status'));
    }

    // To create order view
    public function createOrder($status = []){
        $prod  = $this->setProductDetails();
        $jsonProductDetails = json_encode($prod); 
        $status = json_encode($status);
        return view('order',compact('jsonProductDetails'),compact('status'));
    }

    // To store the order in shopify
    public function saveOrder(DraftOrderValidation $req) {
        $api = new BasicShopifyAPI();
        $password   = Config::get('shopify.password');
        $shopDomain = Config::get('shopify.shopDomain');
        $api->setShop($shopDomain);
        $api->setAccessToken($password);
        $setOrderRequest = $this->setOrderRequest($req);
        $params['order'] = $setOrderRequest;
        $result = $api->rest('POST', htmlentities('/admin/api/'.$this->shopify_api_version.'/orders.json'), $params); 
        
        $prod  = $this->setProductDetails();
        $jsonProductDetails = json_encode($prod);
        return redirect()->action('DraftOrderController@createOrder',['status' => 'success']);
    }


    // To store draft order in shopify
    public function create(DraftOrderValidation $req) {
        $api = new BasicShopifyAPI();
        $password   = Config::get('shopify.password');
        $shopDomain = Config::get('shopify.shopDomain');
        $api->setShop($shopDomain);
        $api->setAccessToken($password);
        $setDraftOrderRequest = $this->setOrderRequest($req);
        $params['draft_order'] = $setDraftOrderRequest;
        $result = $api->rest('post', htmlentities('/admin/api/'.$this->shopify_api_version.'/draft_orders.json'), $params); 
        $prod  = $this->setProductDetails();
        $jsonProductDetails = json_encode($prod);
        return redirect()->action('DraftOrderController@createDraftOrder',['status' => 'success']);
    }

    // To set the request for order and draft order
    public function setOrderRequest($reqObj){
        
        $validated = $reqObj->validated();
        $request = [];
        $lineProperTies = [];
        $prod  = $this->setProductDetails();
        $i = 1;
        foreach ($reqObj['prod-id'] as $key => $productId){
            $propertyValue = '';
            if($i < 10){
                $propertyKey = 'Vitamins0'.$i++;
            }else if($i >= 10){
                $propertyKey = 'Vitamins'.$i++;
            }
            if (isset($prod['products'][$productId])){

            if (isset($reqObj['prod-variant-id'][$key])){
                 $variantId = $reqObj['prod-variant-id'][$key]; 

                if (isset($reqObj['prod-qty'][$key])){

                    $propertyValue .= $prod['products'][$productId]['title'] . ' | '. 
                                $prod["products"][$productId]['variants'][$variantId]['title']. ' | ' .  $reqObj['prod-qty'][$key] . ' | ' . 
                                $prod["products"][$productId]['variants'][$variantId]['id']. ' | ' . $prod["products"][$productId]['variants'][$variantId]['sku'];
                    $lineProperTies[] = ['name'=>$propertyKey,'value'=>$propertyValue];
                    
                }

            }
                
        }

    }
    $request['customer']['first_name'] = $reqObj->firstName;
    $request['customer']['last_name'] = $reqObj->lastName;
    $request['customer']['email'] = $reqObj->email;
    $request['email'] = $reqObj->email;
    $request['line_items'] = array(
        array(
            'title' => "Your Monthly Subscription",
            'product_id' => '4359186350162',
            'variant_id' => '31228232728658',
            'quantity' => '1',
            'properties' => $lineProperTies
        )
    );
    $request['shipping_address'] = array(
            "address1" => $reqObj->shipAddress1,
            "address2"=> $reqObj->shipAddress2,
            "city"=> $reqObj->shipCity,
            "country"=> $reqObj->shipCountry,
            "first_name"=> $reqObj->shipFirstname,
            "lat_name"=> $reqObj->shipLastname,
            "province"=> $reqObj->shipProvince,
            "zip"=> $reqObj->shipZip,
        );
    $request['billing_address'] = array(
            "address1" => $reqObj->billAddress1,
            "address2"=> $reqObj->billAddress2,
            "city"=> $reqObj->billCity,
            "country"=> $reqObj->billCountry,
            "first_name"=> $reqObj->billFirstname,
            "last_name"=> $reqObj->billLastname,
            "province"=> $reqObj->billProvince,
            "zip"=> $reqObj->billZip,
    );
    return $request;
    }

    // To set the shopify product details
    private function setProductDetails() {

        $productDetails = $this->productObj->getShopifyProducts();
        $prod  = [];
        if (!empty($productDetails)) {
            $prod['products'] = [];
            $subscription_product = array('1809168695367','4359186350162');
            foreach ($productDetails['products'] as $products) {
                if(in_array($products['id'],$subscription_product)){
                    continue;
                }
                $vari = [];
                foreach ($products['variants'] as $variants) {
                    $vari[$variants['id']] = ['id' => $variants['id'] ,'product_id'=> $variants['product_id'],"title" => $variants['title'],'sku' => $variants['sku']];
                }
                $prod['products'][$products['id']] = ['id'=>$products['id'], 'title'=>$products['title'],'variants'=>$vari];
            }
        }
        return $prod;
    }
}
?>