<?php

namespace App\Http\Controllers;

use DB;
use Log;
use Illuminate\Http\Request;
use Config;
use App\Http\Controllers\QuizController;
use App\Http\ResponseHelper;
use App\Model\IterableApi;

class MailController extends Controller
{
    public $iterableObj;

    public function __construct()
    {
        $this->helper = new ResponseHelper();
        $this->campaignId = env('CAMPAIGN_ID', true);

        $this->iterableObj = new IterableApi();
    }

    public function sendMail(){

        Log::channel('email')->info("Cron Strted-----------------");

        $controller = new QuizController();
        $userQuizDetails = DB::table('quiz_user')
        ->select('created_at', 'recommended_at', 'mail_sent', 'session_id','id')
        ->where([
            ['mail_sent', '=', '0'],
            ['recommended_at', '=', NULL]
        ])->get();

        if(!empty($userQuizDetails))
        {
            $updateIDs = array();
            foreach($userQuizDetails as $userQuiz){

                $email = DB::table('users')->select('user_email')->where([
                    'id' => $userQuiz->id
                ])->value('user_email');
                // $email =  "testing@p80w.com";
            
                $quizStartTime =  $userQuiz->created_at; 
                $recommendedAt =  $userQuiz->recommended_at;             
                $currentTime   =  date('Y-m-d H:i:s');
                $sentMail      =  $userQuiz->mail_sent;
                $sessionID     =  $userQuiz->session_id;
                
                $timeDiff = $this->getTimeDiff($quizStartTime, $currentTime);

                // Send Mail if difference is 1 hour and mail is not sent earlier
                if($timeDiff >= 1 && $sentMail !== 1)
                {
                    $eventRequest = array(
                        "email" => $email, 
                        "eventName" => "Quiz Abandonment"
                        // "campaignId" => $this->campaignId           
                    );
                     // Trigger Event
                    $eventResponse = $controller->createEvent(json_encode($eventRequest));

                    if(isset($eventResponse->code) && $eventResponse->code == 'Success'){
                        Log::channel('email')->info('Quiz Abandonment Event triggered successfully Please check mail : '.$email);
                        $returnData = [
                            'stored' => true,
                            'message' => 'success'
                        ];

                        array_push($updateIDs, $sessionID); // To update the sent_email to 1
                    }
                    else{
                        Log::channel('email')->info('Event not triggered');
                        $returnData = [
                            'error' => 'Event not triggered'
                        ];
                    }

                }
                else{
                    Log::channel('email')->info('No need to send mail notification to the user '.$email.' with sessionID ='.$sessionID);
                }
                
            }

            $updateQuizUser = DB::table('quiz_user')
                ->whereIn('session_id', $updateIDs)
                ->update(['mail_sent' => 1]);
            
            echo"done";die;
        }


    }

    public function getTimeDiff($date1, $date2){

        $diff = abs(strtotime($date2) - strtotime($date1));  

        $years = floor($diff / (365*60*60*24));

        $months = floor(($diff - $years * 365*60*60*24) 
                                    / (30*60*60*24));  

        $days = floor(($diff - $years * 365*60*60*24 -  
                    $months*30*60*60*24)/ (60*60*24)); 

        $hours = floor(($diff - $years * 365*60*60*24  
        - $months*30*60*60*24 - $days*60*60*24) / (60*60)); 

        return $hours;
    }

    public function sendProspectNotification(Request $request){

        $controller = new QuizController();
        $email = $request->input('email');

        if(empty($email))
        {
            $returnData = [
                'error' => 'Email is empty'
            ];

            return $this->helper->returnData(600, "error", $returnData );
        }

        $userIterableUpdateRequest = array(
            "email" => $email,
            "dataFields" => array(
                "accepts_marketing" => true
            )
        );
        
        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));
        
        if (isset($userIterableUpdateResponse->code) && strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            Log::channel('newsLetterEmailLog')->info("Successfully updated the accept marketing properties for Email (".
            strtolower($email).")");
        }else{
            
            Log::channel('newsLetterEmailLog')->info("Failed to update the accept marketing for Email (".
            strtolower($email).")");
            
            Log::channel('newsLetterEmailLog')->info(json_encode($userIterableUpdateResponse));
           
        }

        $eventRequest = array(
            "email" => $email, 
            "eventName" => "Prospect"
            // "campaignId" => $this->campaignId           
        );
         // Trigger Event
        $eventResponse = $controller->createEvent(json_encode($eventRequest));

        if(isset($eventResponse->code) && $eventResponse->code == 'Success'){
            Log::channel('newsLetterEmailLog')->info('status => Event triggered successfully Please check mail : '.$email);
            $returnData = [
                            'stored' => true                            
                        ];
            return $this->helper->returnData(200, "success", $returnData );
        }
        else{
            Log::channel('newsLetterEmailLog')->info('status =>Event not triggered');

            $returnData = [
                            'error' => 'Event not triggered'
                        ];
            return $this->helper->returnData(600, "error", $returnData );
        }

    }   

}
