<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use Log;
use App\Model\Shopify\Customer;
use App\Model\RechargeApi;
use App\Model\IterableApi;
use App\Helper\Helper;

class RechargeCustomerHashUpdateToMetaCron extends Controller
{

    private $rechargeObj;
    private $iterableObj;
    private $processedSubscriptions;
    private $customerObj;
    private $helperObj;
    private $customerEmail;
    private $rechargeCustomerId;
    private $rechargeSubscriptionId;
    private $metaFieldAllProcessedReport;
    private $metaFieldUpdatedReport;
    private $metaCustomerProcessedCount = [];

    public function __construct()
    {          
        $this->processedSubscriptions = [];
        $this->customerObj = new Customer();
        $this->rechargeObj   = new RechargeApi();
        $this->iterableObj = new IterableApi();  
        $this->helperObj = new Helper();

        date_default_timezone_set('America/Los_Angeles');

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../public/subscriptionMetaUpdatedReport/";
    }

    public function getLogFilesList() {
        
        $this->reportFileDirectory = date('dmY');

        $this->logPath = $this->logDirectory . $this->reportFileDirectory ;
        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }
        
        $this->metaFieldAllProcessedReport = "subscription-meta-processed-list-".date("Y_m_d_H_i").".csv";
        $this->metaFieldUpdatedReport = "subscription-meta-update-list-".date("Y_m_d_H_i").".csv";

        $this->logFiles = array(
            'metaFieldUpdatedCustomers' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->metaFieldUpdatedReport,
                "mode"     => "",
                "columns"  => array(
                    "Email",
                    "ShopifyCustomerID",
                    "RechargeCustomerID",
                    "SubscriptionID",
                    "MetaFieldValue",                    
                    "Status"
                )
            ),
            'totalProcessedCustomers' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->metaFieldAllProcessedReport,
                "mode"     => "",
                "columns"  => array(
                    "Email",
                    "ShopifyCustomerID",
                    "RechargeCustomerID",
                    "SubscriptionID"
                )
            )        
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {

            foreach ($this->logFiles as $logType=>$fileDetails){

                if ($fileDetails['type']=="csv"){
                    $mode = !empty($fileDetails['mode']) ? $fileDetails['mode'] : "w";

                    if ($mode != "r"){
                        $fileName    = $fileDetails['filename'];
                        $filePointer = fopen($fileName,$mode);
                        $columns     = $fileDetails['columns'];
                        fputcsv($filePointer,$columns);
                        chmod($fileName, 0777);
                        fclose($filePointer);
                    }
                }
            }
        }
    }
    
    /**
     * Processing metaField Update
     */
    public function processMetaFieldUpdate() {

        $fileNameAll    = $this->logFiles['totalProcessedCustomers']['filename'];
        $filePointerAll = fopen($fileNameAll, "a");
        $startDate = new Carbon(date('Y-m-d'), 'America/Los_Angeles');
        $startDate = $startDate->subDays("1");
        $startDate = $startDate->format('Y-m-d');
        $allSubscriptionsByDate = $this->getAllSubscriptionsByDate($startDate);

        if(isset($allSubscriptionsByDate) && !empty($allSubscriptionsByDate)) {

            foreach($allSubscriptionsByDate['subscriptions'] as $subscription){

                $shopifyCustomerID = '';
                $rechargeCustomerHash = '';
                $this->customerEmail = '';
                $this->rechargeCustomerId = '';
                $this->rechargeSubscriptionId = '';

                $customerID = trim($subscription['customer_id']);
                $rechargeCustomerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerID);
                if (!isset($rechargeCustomerDetails) || empty($rechargeCustomerDetails) || 
                !empty($rechargeCustomerDetails->error) || !empty($rechargeCustomerDetails->errors)) {

                    if (!isset($rechargeCustomerDetails) || empty($rechargeCustomerDetails)) {

                        Log::channel('subscriptionMetaUpdate')->info("(Fetch recharge customer details): Failed to get 
                        the recharge customer details (Email:".$subscription['email']).")";
                    } elseif (!empty($rechargeCustomerDetails->error) || !empty($rechargeCustomerDetails->errors)) {

                        $errorMessage = !empty($rechargeCustomerDetails->error) ? $rechargeCustomerDetails->error : 
                        $rechargeCustomerDetails->errors;
                        Log::channel('subscriptionMetaUpdate')->info("(Fetch recharge customer details): Error: ".
                        json_encode($errorMessage));
                    }
                    continue;
                }
                $shopifyCustomerID = $rechargeCustomerDetails->customer->shopify_customer_id;
                $rechargeCustomerHash = $rechargeCustomerDetails->customer->hash;
                $this->customerEmail = $subscription['email'];
                $this->rechargeCustomerId = $subscription['customer_id'];
                $this->rechargeSubscriptionId = $subscription['id'];

                $processingCustomer = array(
                    trim($this->customerEmail),
                    $shopifyCustomerID,
                    $this->rechargeCustomerId,
                    $this->rechargeSubscriptionId,                
                );
                fputcsv($filePointerAll, $processingCustomer);
                $this->metaCustomerProcessedCount[] = $this->customerEmail;
                $this->processUpdate($shopifyCustomerID, $rechargeCustomerHash);
            }
        }
  
        // for sending the report email 
        $this->sendReportForSubscriptionMetaFieldCreation();
    }

    /**
     * Checking with the Shopify subscription metaField 
     */
    public function processUpdate($shopifyCustomerID, $rechargeCustomerHash) {

        $shopifyCustomerMetaFields = $this->customerObj->getCustomerMetafields($shopifyCustomerID);
        if (isset($shopifyCustomerMetaFields['error']) || isset($shopifyCustomerMetaFields['errors'])) {

            if (isset($shopifyCustomerMetaFields['error'])) {

                $returnData = [
                    'error' => json_encode($shopifyCustomerMetaFields['error'])
                ];
            } elseif (isset($shopifyCustomerMetaFields['errors'])) {
                $returnData = [
                    'error' => json_encode($shopifyCustomerMetaFields['errors'])
                ];
            } else {
                $returnData = [
                    'error' => 'Failed to get the subscription metaField',
                ];
            }
            Log::channel('subscriptionMetaUpdate')->info("(Fetch shopify customer metaFields): Error: ".
            json_encode($returnData));
            return;
        }
        $subscriptionMetaFieldStatus = 0;
        foreach ($shopifyCustomerMetaFields['metafields'] as $metaFields) {
            if (trim($metaFields['key']) == 'customer_string' && trim($metaFields['namespace']) == 'subscriptions') {
                $subscriptionMetaFieldStatus = 1;
                break;
            } else {
                continue;
            }
        }
        if ($subscriptionMetaFieldStatus == 1) {

            Log::channel('subscriptionMetaUpdate')->info("(subscription customer metaField): Subscription Shopify 
            metaField is already exist, shCustomerID: ".$shopifyCustomerID);
        }
        if ($subscriptionMetaFieldStatus == 0) {
            $this->createSubscriptionMetaField($shopifyCustomerID, $rechargeCustomerHash);
        }
    }

    /**
     * Create subscription metaField in Shopify customer
     */
    public function createSubscriptionMetaField($shopifyCustomerID, $rechargeCustomerHash) {

        $fileName    = $this->logFiles['metaFieldUpdatedCustomers']['filename'];
        $filePointer = fopen($fileName, "a");
        $metaCreateRequest = array(
            "id" => $shopifyCustomerID,
            "metafields" => array(
                array(
                    "namespace" => "subscriptions",
                    "key" => "customer_string",
                    "value" => $rechargeCustomerHash,
                    "value_type" => "string",
                ),
            ),
        );
        $metaCreateResponse = $this->customerObj->createCustomerMetaField($shopifyCustomerID,
            array("customer" => $metaCreateRequest));
        if (isset($metaCreateResponse['error']) || !empty($metaCreateResponse['errors'])) {

            $errorMessage = '';
            if (isset($metaCreateResponse['error'])) {

                $errorMessage = json_encode($metaCreateResponse['error']);
            } elseif (isset($metaCreateResponse['errors'])) {

                $errorMessage = json_encode($metaCreateResponse['errors']);
            } else {

                $errorMessage = 'An error occurred while creating subscription metaField';
            }
            $failedCharge =array(
                trim($this->customerEmail),
                $shopifyCustomerID,
                $this->rechargeCustomerId,
                $this->rechargeSubscriptionId,                
                $rechargeCustomerHash,
                'Failed to create the metaField - '.json_encode($errorMessage)
            );
            fputcsv($filePointer, $failedCharge);
            $this->processedSubscriptions[] = $failedCharge;

            Log::channel('subscriptionMetaUpdate')->info(
                "Failed to create the subscription metaField: ".
                " | ShopifyCustomerId -> ".$shopifyCustomerID.
                " | RechargeHash: ".$rechargeCustomerHash.
                " | Error -> ".json_encode($errorMessage)
            );
        } else {
            $successCharge = array(
                trim($this->customerEmail),
                $shopifyCustomerID,
                $this->rechargeCustomerId,
                $this->rechargeSubscriptionId,                
                $rechargeCustomerHash,
                'Success'
            );
            fputcsv($filePointer, $successCharge);


            $this->processedSubscriptions[] = $successCharge;
            Log::channel('subscriptionMetaUpdate')->info(
                "Successfully created the subscription metaField: ".
                " | ShopifyCustomerId -> ".$shopifyCustomerID.
                " | RechargeHash: ".$rechargeCustomerHash.
                " | Success"
            );
        }
    }

    /**
     * Send report
     */
    public function sendReportForSubscriptionMetaFieldCreation() {

        $iterableMailRequest = '';
        $reportSendingTime =  new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');        
        $reportSendingTime = $reportSendingTime->toDayDateTimeString();
        $metaFieldTotalProcessedCount = 0;
        $metaFieldUpdatedCount = count($this->processedSubscriptions);
        $metaDifferenceWithTotalCount = 0;

        //$recipientEmails = 'testing@p80w.com';
        $recipientEmails = 'mikegordon79@gmail.com';

        if (count($this->metaCustomerProcessedCount) > 0) {

            $metaFieldTotalProcessedCount = count($this->metaCustomerProcessedCount);
            $metaDifferenceWithTotalCount = $metaFieldTotalProcessedCount - $metaFieldUpdatedCount;
        }

        // Processing subscription meta field update report

        $subject = "Nurish-Dev Shopify Account Integrity checker - ".$reportSendingTime;
        
        if (count($this->processedSubscriptions) == 0) {
            Log::channel('subscriptionMetaUpdate')->info("No subscription metafields were updated to customers during this execution");
            $commonContent = '<p>No Shopify Customers have needed to have their subscription metafields updated</p><br>';

            if ($metaFieldTotalProcessedCount == 1) {
                $commonContent .= $metaFieldTotalProcessedCount.' record processed';
            } else {
                $commonContent .= $metaFieldTotalProcessedCount.' records processed';
            }

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_METAFIELD_CREATE_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => '',
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        } elseif( count($this->processedSubscriptions) > 30) {

            $fileUrl = url('/').'/subscriptionMetaUpdatedReport/'.$this->reportFileDirectory.'/'.$this->metaFieldUpdatedReport;
            $commonContent = '<p>'.$metaFieldUpdatedCount.' Shopify Customers have had their subscription metafields successfully updated</p><br>';
            $recordDetails = 'Please <a href="'.$fileUrl.'">click here</a> to download the file.';

            if ($metaFieldTotalProcessedCount == 1) {
                $recordDetails .= '<p>'.$metaFieldTotalProcessedCount.' record processed</p>';
            } else {
                $recordDetails .= '<p>'.$metaFieldTotalProcessedCount.' records processed</p>';
            }

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_METAFIELD_CREATE_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
            
        } elseif (count($this->processedSubscriptions) > 0 && count($this->processedSubscriptions) <= 30) {
            $commonContent = '<p>'.$metaFieldUpdatedCount.' Shopify Customers have had their subscription metafields successfully updated</p><br>'; 
            
            $recordDetails = '';
            if(!empty($this->processedSubscriptions)){           
                $recordDetails .= "
                <table>
                    <tr>
                        <td>#</td>                
                        <td>Email</td>
                        <td>ShopifyCustomerID</td> 
                        <td>RechargeCustomerID</td> 
                        <td>SubscriptionID</td>
                        <td>MetaFieldValue</td>   
                        <td>Status</td>                       
                    </tr>            
                ";

                $i = 1;
                foreach($this->processedSubscriptions as $record){
                    
                    $orderScheduledAtDate =  new Carbon($record[3],'America/Los_Angeles');                            
                    $orderScheduledAtDate = $orderScheduledAtDate->toDayDateTimeString();
                            
                    $recordDetails .= "
                    <tr>
                        <td>".$i."</td>                             
                        <td>".$record[0]."</td> 
                        <td>".$record[1]."</td>  
                        <td>".$record[2]."</td>
                        <td>".$record[3]."</td>
                        <td>".$record[4]."</td>
                        <td>".$record[5]."</td> 
                    </tr>
                    ";
                    $i++;            
                }
                $recordDetails .= '</table>';

                if ($metaFieldTotalProcessedCount == 1) {
                    $recordDetails .= '<p>'.$metaFieldTotalProcessedCount.' record processed</p>';
                } else {
                    $recordDetails .= '<p>'.$metaFieldTotalProcessedCount.' records processed</p>';
                }
            }
            
            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_METAFIELD_CREATE_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        }
        
        if(!empty($iterableMailRequest)) {
            
            $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

            if (strtolower(trim($mailTriggerResponse->code)) == 'success') {           
                
                Log::channel('subscriptionMetaUpdate')->info("subscription metafield update report 
                mail send.........................");
    
            } else {
                
                Log::channel('subscriptionMetaUpdate')->info('subscription metafield update report 
                mail not send......................');
                Log::channel('subscriptionMetaUpdate')->info(json_encode($mailTriggerResponse));
                $error = $mailTriggerResponse->code;
    
                if (empty($error)) {
    
                    $returnData = [
                        'error' => 'failed to send mail(' . $recipientEmails . ')',
                    ];
                } else {
    
                    $returnData = [
                        'error' => $error . ' (' . $recipientEmails . ')',
                    ];
                }
                
                Log::channel('subscriptionMetaUpdate')->info($returnData['error']);
                 
            }
        }
    }
    
    /**
     * Function to get all the recharge subscriptions
     */
    function getAllSubscriptionsByDate($minDate) {

        $subscriptionDetails = array();
        $pageNo = 1;
        $limit  = 200;
        do {
            try{
                //$this->helperObj->printMe("Read recharge subscription records page -". $pageNo);
                $subscriptions = $this->rechargeObj->getAllSubscriptionsByMinDate($minDate, $pageNo, $limit);
                sleep(1);
            }
            catch (Exception $e){
                $errorText =
                "Error form recharge when fetching the subscriptions details and error is -  "
                .$e->getMessage();
                die;
            }
            if (!empty($subscriptions) && isset($subscriptions)){

                $subscriptionData = $this->helperObj->objectToArray($subscriptions);

                foreach ($subscriptionData['subscriptions'] as $subscription){

                    if (!empty($subscription['id'])){

                        $subscriptionDetails[] = $subscription;
                    }
                }
                $subscriptionCount = count($subscriptionData['subscriptions']);
            } else {
                break;
            }

        $pageNo++;

        } while($subscriptionCount == $limit);

        $rechargeSubscriptionDetails['subscriptions'] = $subscriptionDetails;

        return $rechargeSubscriptionDetails;
    }
}
