<?php

namespace App\Http\Controllers;

use DB;
use Log;
use App\Users;
use Illuminate\Http\Request;
use App\Http\ResponseHelper;
use App\Jobs\ShipmentTrackingUpdateJob;
use Config;
use OhMyBrew\BasicShopifyAPI;

class ShipmentTrackingWebhookController extends Controller
{

    private $helper;
    private $shopifyProductDao;
    private $sharedSecret;
    private $randomString;
    private $shopify_api_version;

    public function __construct()
    {
        $this->helper = new ResponseHelper();
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    function processWebhook()
    {
        
        $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_shopify_webhook($requestData, $hmac_header);
        $webhookStatus = var_export($verified, true);
        
        if ($webhookStatus) {

            $receivedData = json_decode($requestData);
            
            Log::channel('shipmentWebhookIterable')->info("Shopify customer creation webhook validation success");
            
            Log::channel('shipmentWebhookIterable')->info($requestData);

            if(empty($receivedData->email)) {
                
                Log::channel('shipmentWebhookIterable')->info("Error in the iterable user profile tacking update -
                Missing required parameters in the webhook response or the response is not correct");
                
            } else {
                
                Log::channel('shipmentWebhookIterable')->info("data transfered to ShipmentTrackingUpdateJob.");
                $queueResponse = dispatch(new ShipmentTrackingUpdateJob($receivedData));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
                
            }
        } else {
            Log::channel('shipmentWebhookIterable')->info("Shopify customer creation webhook validation failed".$requestData);
        }

    }
    
    public function verify_shopify_webhook($data, $hmac_header) {

        $this->sharedSecret   = Config::get('shopify.secretShipmentTrack');
        // $this->sharedSecret   = '4a174fb55065c47c6838c7802340495b2acda50370ff94dcae1ad2556871b455';
        
        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, $this->sharedSecret, true));
        return hash_equals($hmac_header, $calculated_hmac);
    }


}
