<?php

namespace App\Http\Controllers;

use Log;
use Carbon\Carbon;
use App\Model\RechargeApi;
use App\Model\Shopify\Customer;
use App\Model\Shopify\Product;
use App\Model\Shopify\Order;
use App\Model\IterableApi;
use App\Helper\Helper;
use Illuminate\Http\Request;

class giftPackController extends Controller {

    private $rechargeObj;
    private $helperObj;
    private $productObj;
    private $orderObj;
    private $customerObj;
    private $iterableObj;
    private $gifteeProcessedEvents = [];
    private $giftExpirationProcessedEvents = [];
    private $logDirectory;
    private $reportFileDirectory;
    private $logPath;
    private $gifteeEventsCreatedReport;
    private $giftExpirationEventsCreatedReport;
    private $logFiles;
    
    public function __construct() {

        $this->rechargeObj = new RechargeApi();
        $this->productObj = new Product();
        $this->orderObj = new Order();
        $this->customerObj = new Customer();
        $this->iterableObj = new IterableApi();
        //date_default_timezone_set('America/Los_Angeles');
    }

    /**
     * Update ship date tag to the order
     */
    public function setGiftRecipientOrderShipDate(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }

        $receivedData = json_decode($requestData);

        if (empty($receivedData->subscriptionId) && empty($receivedData->shipDate)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }

        $subscriptionId = !empty($receivedData->subscriptionId) ? trim($receivedData->subscriptionId) : '';
        $shipDate = !empty($receivedData->shipDate) ? trim($receivedData->shipDate) : '';
        $newPassword = !empty($receivedData->newPassword) ? trim($receivedData->newPassword) : '';
        $confirmPassword = !empty($receivedData->confirmPassword) ? trim($receivedData->confirmPassword) : '';

        if (empty($subscriptionId)) {

            $returnData = [
                'error' => 'Required parameter subscriptionId is missing in the endpoint request',
            ];
            Log::channel('giftPackLog')->info("Required parameter subscriptionId is missing 
            in the endpoint request, endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        if (empty($shipDate)) {

            $returnData = [
                'error' => 'Required parameter shipDate is missing in the endpoint request',
            ];
            Log::channel('giftPackLog')->info("Required parameter shipDate is missing 
            in the endpoint request, endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        if (!empty($newPassword) || !empty($confirmPassword)) {

            if ($newPassword != $confirmPassword) {

                $returnData = [
                    'error' => 'Found password mismatch',
                ];
                Log::channel('giftPackLog')->info("Found password mismatch, 
                endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }
        }

        // Get charge details from recharge by subscriptionId
        $chargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionId($subscriptionId);
        if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
            empty($chargeDetails) || empty($chargeDetails->charges)) {

            $rechargeError  = "Unknown error";
            if (!empty($chargeDetails->error)) {

                $rechargeError = json_encode($chargeDetails->error);
            } else if (!empty($chargeDetails->errors)) {

                $rechargeError = json_encode($chargeDetails->errors);
            } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                $rechargeError = 'Failed to get the charge details from recharge';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("Failed to get the charge details from 
            the recharge(subscriptionId:".$subscriptionId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $shopifyOrderId = !empty($chargeDetails->charges[0]->shopify_order_id) ? 
        $chargeDetails->charges[0]->shopify_order_id : '';

        if (empty($shopifyOrderId)) {

            $returnData = [
                'error' => 'Failed to get the order details from Shopify',
            ];
            Log::channel('giftPackLog')->info("Failed to get the Shopify orderId from recharge charge, 
            subscriptionId: ".$subscriptionId);
            return $this->returnData(600, "error", $returnData);
        }

        // Get order details from Shopify
        $shopifyOrderDetails = $this->orderObj->getOrderDetailsById($shopifyOrderId);
        if (!empty($shopifyOrderDetails->error) || !empty($shopifyOrderDetails->errors) ||
            empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderDetails->error)) {

                $shopifyError = json_encode($shopifyOrderDetails->error);
            } else if (!empty($shopifyOrderDetails->errors)) {

                $shopifyError = json_encode($shopifyOrderDetails->errors);
            } else if (empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

                $shopifyError = 'Failed to get the order details from Shopify';
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('giftPackLog')->info("Failed to get the order details 
            from Shopify(OrderId:".$shopifyOrderId."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $gifteeFirstName = '';
        $gifteeLastName = '';
        $gifteeEmailAddress = '';
        $orderLineItemProperties = !empty($shopifyOrderDetails['order']['line_items'][0]['properties']) ?
        $shopifyOrderDetails['order']['line_items'][0]['properties'] : '';
        if (!empty($orderLineItemProperties)) {

            foreach ($orderLineItemProperties as $propertyKey => $propertyValue) {

                switch($propertyValue['name']) {
                    
                    case 1:
                        $gifteeFirstName = !empty($propertyValue['name'] == 'giftee_first_name') ? 
                        trim($propertyValue['value']) : '';
                    break;

                    case 2:
                        $gifteeLastName = !empty($propertyValue['name'] == 'giftee_last_name') ? 
                        trim($propertyValue['value']) : '';
                    break;

                    case 3:
                        $gifteeEmailAddress = !empty($propertyValue['name'] == 'giftee_email') ? 
                        trim($propertyValue['value']) : '';
                    break;
                    
                }
            }
        }

        // Set customer password
        if (!empty($newPassword) && !empty($confirmPassword)) {

            if (empty($gifteeEmailAddress)) {
                
                $returnData = [
                    'error' => 'Failed to find the giftee email from the Shopify order',
                ];
                Log::channel('giftPackLog')->info("Failed to find the giftee email from the Shopify order, 
                subscriptionId: ".$subscriptionId.", orderId:".$shopifyOrderId);
                return $this->returnData(600, "error", $returnData);
            }
            $customerCreateRequest = [];
            $customerCreateRequest = array(
                "customer" => array(
                    "first_name" => $gifteeFirstName,
                    "last_name" => $gifteeLastName,
                    "email" => $gifteeEmailAddress,
                    "password" => $newPassword,
                    "password_confirmation" => $confirmPassword,
                    "send_email_welcome" => false
                ),
            );

            // Create giftee Shopify account
            $shopifyCustomerCreateResponse = $this->orderObj->createShopifyCustomer($customerCreateRequest);

            if (!empty($shopifyCustomerCreateResponse->error) || !empty($shopifyCustomerCreateResponse->errors) ||
                empty($shopifyCustomerCreateResponse)) {

                $shopifyError  = "Unknown error";
                if (!empty($shopifyCustomerCreateResponse->error)) {

                    $shopifyError = json_encode($shopifyCustomerCreateResponse->error);
                } else if (!empty($shopifyCustomerCreateResponse->errors)) {

                    $shopifyError = json_encode($shopifyCustomerCreateResponse->errors);
                } else if (empty($shopifyCustomerCreateResponse)) {

                    $shopifyError = 'Failed to create the giftee account in Shopify';
                }
                $returnData = [
                    'error' => $shopifyError
                ];

                Log::channel('giftPackLog')->info("Failed to create the giftee account in Shopify(customerCreateRequest:".
                json_encode($customerCreateRequest)."), Error: (".$shopifyError.")");
                return $this->returnData(600, "error", $returnData);
            }
        }

        // Update Shopify order tag
        $orderTags = !empty($shopifyOrderDetails['order']['tags']) ? 
        trim($shopifyOrderDetails['order']['tags']) : '';

        if (empty($orderTags)) {

            $returnData = [
                'error' => 'The Shopify order tags are empty',
            ];
            Log::channel('giftPackLog')->info("The Shopify order tags are empty, 
            subscriptionId: ".$subscriptionId.", orderId:".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        $shOrderTags = explode(",", $orderTags);
        $shOrderTags = array_map('trim', $explodeTags);
        if (!in_array('unclaimed', $shOrderTags)) {

            $returnData = [
                'error' => 'Not found unclaimed tag in the order',
            ];
            Log::channel('giftPackLog')->info("Not found unclaimed tag in the order, 
            subscriptionId: ".$subscriptionId.", orderId:".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        $updatedShopifyTags = '';
        if (!empty($orderTags)) {

            $updatedShopifyTags = $this->checkAndModifyShopifyOrderTags($orderTags, $shipDate);
        }

        $orderUpdateRequest = [];
        $orderUpdateRequest = array (
            "order" => array (
                "id" => $shopifyOrderId,
                "tags" => $updatedShopifyTags
            )
        );

        // Get order details from Shopify
        $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $shopifyOrderId);
        if (!empty($shopifyOrderUpdateResponse->error) || !empty($shopifyOrderUpdateResponse->errors) ||
            empty($shopifyOrderUpdateResponse)) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderUpdateResponse->error)) {

                $shopifyError = json_encode($shopifyOrderUpdateResponse->error);
            } else if (!empty($shopifyOrderUpdateResponse->errors)) {

                $shopifyError = json_encode($shopifyOrderUpdateResponse->errors);
            } else if (empty($shopifyOrderUpdateResponse)) {

                $shopifyError = 'Failed to update the Shopify order';
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('giftPackLog')->info("Failed to update the Shopify order(orderUpdateRequest:".
            json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $responseMessage = 'Successfully updated the order tags Shopify';
        if ($gifteeAccountCreateStatus == 1) {

            $responseMessage = 'Successfully created the giftee account and updated the order tags in Shopify';
        }
        $returnData = [
            'message' => $responseMessage
        ];

        Log::channel('giftPackLog')->info($responseMessage);
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Check and modify Shopify order tags
     */
    public function checkAndModifyShopifyOrderTags($orderTags, $shipDate) {

        $explodeTags = explode(",", $orderTags);
        $explodeTags = array_map('trim', $explodeTags);
        //$explodeTags = array_map('strtolower', $explodeTags);
        foreach ($explodeTags as $tagKey => $tagValue) {

            if (strtolower(trim($tagValue)) == 'unclaimed') {
                unset($explodeTags[$tagKey]);
            }
        }
        $explodeTags[] = 'claimed';
        $explodeTags[] = 'shipDate:'.$shipDate;
        $implodedShopifyTags = implode(',', $explodeTags);
        return $implodedShopifyTags;
    }

    /**
     * Get all subscriptions(regular and gift subscriptions if any)
     */
    public function getAllSubscriptions(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }

        $receivedData = json_decode($requestData);

        if ((empty($receivedData->customerId) && empty($receivedData->hash)) || 
        (empty($receivedData->subscriptionId) && empty($receivedData->email))) {

            $returnData = [
                'error' => 'Invalid endpoint request'
            ];            
            Log::channel('giftPackLog')->info("getAllSubscriptions: Invalid endpoint request, request:".$requestData);
            return $this->returnData(600, "error", $returnData);
        }

        $customerId = !empty($receivedData->customerId) ? trim($receivedData->customerId) : '';
        $customerHash = !empty($receivedData->hash) ? trim($receivedData->hash) : '';
        $subscriptionId = !empty($receivedData->subscriptionId) ? trim($receivedData->subscriptionId) : '';
        $recipientEmail = !empty($receivedData->email) ? trim($receivedData->email) : '';

        $endpointInputIdentifier = '';
        if (!empty($customerId) || !empty($customerHash)) {

            $endpointInputIdentifier = 'customerId';
            if (empty($customerId)) {

                $returnData = [
                    'error' => 'Required parameter customerId is missing in the endpoint request',
                ];
                Log::channel('giftPackLog')->info("getAllSubscriptions: Required parameter customerId is missing 
                in the endpoint request, endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);

            } else if (empty($customerHash)) {

                $returnData = [
                    'error' => 'Required parameter customerHash is missing in the endpoint request',
                ];
                Log::channel('giftPackLog')->info("getAllSubscriptions: Required parameter customerHash is missing 
                in the endpoint request, endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }

        } else if (!empty($subscriptionId) || !empty($recipientEmail)) {

            $endpointInputIdentifier = 'subscriptionId';
            if (empty($subscriptionId)) {

                $returnData = [
                    'error' => 'Required parameter subscriptionId is missing in the endpoint request',
                ];
                Log::channel('giftPackLog')->info("getAllSubscriptions: Required parameter subscriptionId is missing 
                in the endpoint request, endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);

            } else if (empty($recipientEmail)) {

                $returnData = [
                    'error' => 'Required parameter recipientEmail is missing in the endpoint request',
                ];
                Log::channel('giftPackLog')->info("getAllSubscriptions: Required parameter recipientEmail is missing 
                in the endpoint request, endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }
        }
        
        $customerSubscriptionDetails = [];
        if ($endpointInputIdentifier == 'customerId') {

            $customerSubscriptionDetails = $this->getExistingCustomerSubscriptionDetails($receivedData, 
            $customerId, $customerHash);

        } else if ($endpointInputIdentifier == 'subscriptionId') {

            $customerSubscriptionDetails = $this->getNewCustomerSubscriptionDetails($receivedData, $subscriptionId, 
            $recipientEmail);
        }

        $returnData = [
            'message' => 'Successfully get the subscription details',
            'subscriptions' => $customerSubscriptionDetails
        ];

        Log::channel('giftPackLog')->info("getAllSubscriptions: ".json_encode($returnData));
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Get existing customer subscriptions and gift subscriptions if any
     */
    public function getExistingCustomerSubscriptionDetails($receivedData, $customerId, $customerHash) {

        // Get recharge customer details by ID
        $rechargeCustomerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerId);
        if (!empty($rechargeCustomerDetails->error) || !empty($rechargeCustomerDetails->errors) ||
            empty($rechargeCustomerDetails)) {

            $rechargeError  = "Unknown error";
            if (!empty($rechargeCustomerDetails->error)) {

                $rechargeError = json_encode($rechargeCustomerDetails->error);
            } else if (!empty($rechargeCustomerDetails->errors)) {

                $rechargeError = json_encode($rechargeCustomerDetails->errors);
            } else if (empty($rechargeCustomerDetails)) {

                $rechargeError = 'Failed to get the customer details from recharge';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("getAllSubscriptions: Failed to get the customer details from 
            the recharge(customerId:".$customerId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $shopifyCustomerId = trim($rechargeCustomerDetails->customer->shopify_customer_id);
        $rechargeHashToken = trim($rechargeCustomerDetails->customer->hash);

        if ($rechargeHashToken != $customerHash) {

            $returnData = [
                'error' => 'Failed to authenticate the user'
            ];
            Log::channel('giftPackLog')->info("getAllSubscriptions: Failed to authenticate the user
            (The recharge hash tokens are mismatched), endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }
        
        // Get all the subscriptions by customerId
        $subscriptionDetails = $this->rechargeObj->getAllSubscriptionsByCustomerID($customerId);
        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) ||
            empty($subscriptionDetails)) {

            $rechargeError  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {

                $rechargeError = json_encode($subscriptionDetails->error);
            } else if (!empty($subscriptionDetails->errors)) {

                $rechargeError = json_encode($subscriptionDetails->errors);
            } else if (empty($subscriptionDetails)) {

                $rechargeError = 'Failed to get the subscriptions from recharge by customerId';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("getAllSubscriptions: Failed to get the subscriptions from 
            recharge by customerId(customerId:".$customerId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }
        //print_r($subscriptionDetails->subscriptions); die;
        // Get shopify details of a customer
        $shopifyCustomerDetails = $this->customerObj->getCustomerDetails($shopifyCustomerId);
        if (!empty($shopifyCustomerDetails->error) || !empty($shopifyCustomerDetails->errors) ||
            empty($shopifyCustomerDetails)) {

            $shError  = "Unknown error";
            if (!empty($shopifyCustomerDetails->error)) {

                $shError = json_encode($shopifyCustomerDetails->error);
            } else if (!empty($shopifyCustomerDetails->errors)) {

                $shError = json_encode($shopifyCustomerDetails->errors);
            } else if (empty($shopifyCustomerDetails)) {

                $shError = 'Failed to get the customer details from Shopify';
            }
            $returnData = [
                'error' => $shError
            ];

            Log::channel('giftPackLog')->info("getAllSubscriptions: Failed to get the customer details from Shopify
            (shCustomerId:".$shopifyCustomerId."), Error: (".$shError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $allSubscriptions = [];
        $customerTags = !empty($shopifyCustomerDetails['customer']['tags']) ? 
        trim($shopifyCustomerDetails['customer']['tags']) : '';
        $giftSubscriptionIdFromTags = [];
        $giftSubscriptionDetails = [];

        if (!empty($customerTags)) {

            $giftSubscriptionIdFromTags = $this->checkGiftSubscriptionsInTags($customerTags);
            if (!empty($giftSubscriptionIdFromTags)) {

                // Get subscription by ID
                foreach ($giftSubscriptionIdFromTags as $subKey => $subValue) {

                    $giftSubscriptions = $this->rechargeObj->getSubscription($subValue);
                    if (!empty($giftSubscriptions->error) || !empty($giftSubscriptions->errors) ||
                        empty($giftSubscriptions)) {

                        $rechargeError  = "Unknown error";
                        if (!empty($giftSubscriptions->error)) {

                            $rechargeError = json_encode($giftSubscriptions->error);
                        } else if (!empty($giftSubscriptions->errors)) {

                            $rechargeError = json_encode($giftSubscriptions->errors);
                        } else if (empty($giftSubscriptions)) {

                            $rechargeError = 'Failed to get the subscription details by subscriptionId';
                        }
                        $returnData = [
                            'error' => $rechargeError
                        ];

                        Log::channel('giftPackLog')->info("getAllSubscriptions: Failed to get the subscription 
                        details by subscriptionId(subscriptionId:".$subValue."), Error: (".$rechargeError.")");
                        return $this->returnData(600, "error", $returnData);
                    }
                    $giftSubscriptionDetails[] = $giftSubscriptions->subscription;
                    sleep(1);
                }
            }
        }

        if (empty($subscriptionDetails->subscriptions) && empty($giftSubscriptionDetails)) {

            $returnData = [
                'error' => 'No subscription found'
            ];
            Log::channel('giftPackLog')->info("getAllSubscriptions: No subscription found, 
            endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        if (!empty($subscriptionDetails->subscriptions)) {

            $allSubscriptions['regular'] = $subscriptionDetails->subscriptions;
        }
        if (!empty($giftSubscriptionDetails)) {

            $allSubscriptions['gift'] = $giftSubscriptionDetails;
        }

        return $allSubscriptions;
    }

    /**
     * Get subscription details of the new gift recipient
     */
    public function getNewCustomerSubscriptionDetails($receivedData, $subscriptionId, $recipientEmail) {
        
        $giftSubscriptionDetails = [];
        $allSubscriptions = [];
        if (strpos($subscriptionId, ',') !== false) {

            $giftSubscriptionIds = [];

            $explodeSubIds = explode(",", $subscriptionId);
            $giftSubscriptionIds = array_map('trim', $explodeSubIds);

            foreach ($giftSubscriptionIds as $giftSubKey => $giftSubValue) {

                $giftSubscriptions = $this->rechargeObj->getSubscription($giftSubValue);
                if (!empty($giftSubscriptions->error) || !empty($giftSubscriptions->errors) ||
                    empty($giftSubscriptions)) {

                    $rechargeError  = "Unknown error";
                    if (!empty($giftSubscriptions->error)) {

                        $rechargeError = json_encode($giftSubscriptions->error);
                    } else if (!empty($giftSubscriptions->errors)) {

                        $rechargeError = json_encode($giftSubscriptions->errors);
                    } else if (empty($giftSubscriptions)) {

                        $rechargeError = 'Failed to get the subscription details by subscriptionId';
                    }
                    $returnData = [
                        'error' => $rechargeError
                    ];

                    Log::channel('giftPackLog')->info("getAllSubscriptions: Failed to get the subscription 
                    details by subscriptionId(subscriptionId:".$giftSubValue."), Error: (".$rechargeError.")");
                    return $this->returnData(600, "error", $returnData);
                }
                $giftSubscriptionDetails[] = $giftSubscriptions->subscription;
                sleep(1);
            }
        } else {

            $giftSubscriptions = $this->rechargeObj->getSubscription($subscriptionId);
            if (!empty($giftSubscriptions->error) || !empty($giftSubscriptions->errors) ||
                empty($giftSubscriptions)) {

                $rechargeError  = "Unknown error";
                if (!empty($giftSubscriptions->error)) {

                    $rechargeError = json_encode($giftSubscriptions->error);
                } else if (!empty($giftSubscriptions->errors)) {

                    $rechargeError = json_encode($giftSubscriptions->errors);
                } else if (empty($giftSubscriptions)) {

                    $rechargeError = 'Failed to get the subscription details by subscriptionId';
                }
                $returnData = [
                    'error' => $rechargeError
                ];

                Log::channel('giftPackLog')->info("getAllSubscriptions: Failed to get the subscription 
                details by subscriptionId(subscriptionId:".$giftSubValue."), Error: (".$rechargeError.")");
                return $this->returnData(600, "error", $returnData);
            }
            $giftSubscriptionDetails[] = $giftSubscriptions->subscription;
        }

        $allSubscriptions['gift'] = $giftSubscriptionDetails;
        return $allSubscriptions;
    }

    /**
     * Check gift subscription details in the customer tags
     */
    public function checkGiftSubscriptionsInTags($customerTags) {

        $giftSubIds = [];
        $explodeTags = explode(",", $customerTags);
        $explodeTags = array_map('trim', $explodeTags);
        foreach ($explodeTags as $tagKey => $tagValue) {

            if (strpos($tagValue, 'giftSub') !== false) {

                $explodeValue = explode(":", $tagValue);
                $giftSubIds[] = trim($explodeValue[1]);
            }
        }
        return $giftSubIds;
    }

    /**
     * Cron job to create giftee and 30 days remaining iterable events
     */
    public function processGiftSubscriptionEvents() {

        // Create giftee event
        $this->gifteeEventCreation();

        // Create gift_subscription_expiring
        $this->giftExpirationEventCreation();

        // To send the report email 
        $this->sendReportForGiftPackEventCreation();
    }

    /**
     * Giftee event creation in the cron job
     */
    public function gifteeEventCreation() {

        $orders = [];
        $today = date('Y-m-d');
        //$today = '2021-09-25';
        $orderSearchTag = 'giftNotificationDate:'.$today;
        $gifteeEventProcessedTemp = [];
        $shopifyGifteeCustomerId = '';
        $gifteeUrlIfGifteeWithActiveShopifyAccount = 'https://nurish-usd-dev.myshopify.com/account/claimGift';
        $limit  = 200;
        Log::channel('giftCronLog')->info("Started Giftee event creation cron working");
        do {
            try {
                if (!empty($orders['link'])) {

                    $orders = $this->orderObj->getShopifyOrdersByTag($limit, $orders['link'], $orderSearchTag);

                } else {

                    $orders = $this->orderObj->getShopifyOrdersByTag(
                        $limit, NULL, $orderSearchTag);

                    if (empty($orders['body'])) {
                        Log::channel('giftCronLog')->info("No orders found with the ".$orderSearchTag." Tag");
                    }
                }
            }
            catch (Exception $e) {

                Log::channel('giftCronLog')->info("An error occurred while fetching the
                order details from Shopify, Error:".json_encode($e->getMessage()));
                die();
            }
            
            //print_r($orders); die;
            if (!empty($orders) && isset($orders['body'])) {
                
                foreach ($orders['body'] as $order) {

                    if (!empty($order['id'])) {

                        $gifteeFirstName = '';
                        $gifteeLastName = '';
                        $giftNote = '';
                        $giftPurchaserFirstName = '';
                        $giftPurchaserLastName = '';
                        $gifteeActivationLink = '';
                        $giftPackName = '';
                        $giftNumberChargesUntilExpiration = '';
                        $gifteeEmail = '';
                        $token = '';

                        $orderTags = $order['tags'];
                        $existingOrderTags = explode(",", $orderTags);
                        $existingOrderTags = array_map('trim', $existingOrderTags);

                        if (in_array('giftee-notified', $existingOrderTags)) {
                            
                            Log::channel('giftCronLog')->info("Skipped: Already created the Giftee event for the orderId:".
                            $order['id']);
                            continue;
                        }
                        if (!in_array($orderSearchTag, $existingOrderTags)) {

                            Log::channel('giftCronLog')->info("Skipped: Giftee order notification tag not found in the order, 
                            orderId:".$order['id']);
                            continue;
                        }
                        if (empty($order['line_items'][0]['properties'])) {

                            Log::channel('giftCronLog')->info("Skipped: The order line item properties are empty, 
                            orderId:".$order['id']);
                            continue;
                        }
                        foreach ($order['line_items'][0]['properties'] as $lipKey => $lipValue) {

                            switch ($lipValue['name']) {

                                case (trim($lipValue['name']) == 'giftee_first_name'):
                                    $gifteeFirstName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_last_name'):
                                    $gifteeLastName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
            
                                case (trim($lipValue['name']) == 'gift_note'):
                                    $giftNote = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
            
                                case (trim($lipValue['name']) == 'gift_senders_first_name'):
                                    $giftPurchaserFirstName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
            
                                case (trim($lipValue['name']) == 'gift_senders_last_name'):
                                    $giftPurchaserLastName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
            
                                case (trim($lipValue['name']) == 'gift_pack_name'):
                                    $giftPackName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
            
                                case (trim($lipValue['name']) == 'gift_number_charges_until_expiration'):
                                    $giftNumberChargesUntilExpiration = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_email'):
                                    $gifteeEmail = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
                            }
                        }

                        $shopifyOrderId = trim($order['id']);
                        $shopifyGiftPurchaserCustomerId = trim($order['customer']['id']);

                        // Get charge details from recharge by shopify order ID
                        $chargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($shopifyOrderId);
                        if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
                            empty($chargeDetails) || empty($chargeDetails->charges)) {

                            $rechargeError  = "Unknown error";
                            if (!empty($chargeDetails->error)) {

                                $rechargeError = json_encode($chargeDetails->error);
                            } else if (!empty($chargeDetails->errors)) {

                                $rechargeError = json_encode($chargeDetails->errors);
                            } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                                $rechargeError = 'Failed to get the charge details by using the given orderId: '.$shopifyOrderId;
                            }
                            $returnData = [
                                'error' => $rechargeError
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the charge details from 
                            the recharge(OrderId:".$shopifyOrderId."), Error: (".$rechargeError.")");
                            continue;
                        }
                        $purchaserSubscriptionId = !empty($chargeDetails->charges[0]->line_items[0]->subscription_id) ? 
                        trim($chargeDetails->charges[0]->line_items[0]->subscription_id) : '';

                        $purchaserName = $giftPurchaserFirstName. ' ' .$giftPurchaserLastName;

                        $recipientShippingAddress = !empty($chargeDetails->charges[0]->shipping_address) ? 
                        $chargeDetails->charges[0]->shipping_address : '';

                        if (empty($purchaserSubscriptionId) || empty($gifteeEmail)) {

                            Log::channel('giftCronLog')->info("Skipped: The gift purchaser subscriptionId or giftee email is empty
                            (OrderId:".$shopifyOrderId.", SubscriptionId:".$purchaserSubscriptionId.", GifteEmail:".$gifteeEmail.")");
                            continue;
                        }
                        $token = $purchaserSubscriptionId.':'.$gifteeEmail;
                        for ($i = 0; $i < 3; $i++) { 

                            $token = base64_encode($token);
                        }

                        // Check giftee account email exists in Shopify
                        $customerDetails = $this->customerObj->getCustomerDetailBySearch($gifteeEmail);

                        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                            $error = "Unknown error";
                            if (!empty($customerDetails['error'])) {

                                $error = json_encode($customerDetails['error']);
                            } elseif (!empty($customerDetails['errors'])) {
                                
                                $error = json_encode($customerDetails['errors']);
                            }
                            if (empty($error)) {

                            $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $gifteeEmail . ")";
                            }
                            $returnData = [
                                "error" => $error
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the customer details from Shopify(emailAddress:".
                            $gifteeEmail."), Error: (".$error.")");
                            continue;
                        }
                        
                        $gifteeAccountExistsStatus = 0;
                        $existingGiftRecipientDetails = '';
                        if (!empty($customerDetails['customers']) || !empty($customerDetails) || isset($customerDetails['customers'])) {

                            foreach ($customerDetails['customers'] as $customerKey => $customerValue) {

                                if (strtolower(trim($gifteeEmail)) == strtolower(trim($customerValue['email']))) {

                                    $gifteeAccountExistsStatus = 1;
                                    $existingGiftRecipientDetails = $customerValue;
                                    break;
                                }
                            }
                        }
                        
                        if ($gifteeAccountExistsStatus == 1) {

                            $existingCustomerAccountStatus = '';
                            $shopifyGiftRecipientCustomerId = $existingGiftRecipientDetails['id'];
                            $shopifyGifteeCustomerId = $existingGiftRecipientDetails['id'];
                            $existingCustomerAccountStatus = $existingGiftRecipientDetails['state'];
                            if (trim($existingCustomerAccountStatus) != 'enabled') {

                                // Create giftee Shopify account activation_url
                                $customerActivationUrl = $this->customerObj->createAccountActivationLink($shopifyGiftRecipientCustomerId);

                                if (!empty($customerActivationUrl['error']) || !empty($customerActivationUrl['errors']) ||
                                    empty($customerActivationUrl)) {

                                    $shopifyError  = "Unknown error";
                                    if (!empty($customerActivationUrl['error'])) {

                                        $shopifyError = json_encode($customerActivationUrl['error']);
                                    } else if (!empty($customerActivationUrl['errors'])) {

                                        $shopifyError = json_encode($customerActivationUrl['errors']);
                                    } else if (empty($customerActivationUrl)) {

                                        $shopifyError = 'Failed to create the giftee account activation URL';
                                    }
                                    $returnData = [
                                        'error' => $shopifyError
                                    ];

                                    Log::channel('giftCronLog')->info("Failed to create the giftee account activation URL(customerId:".
                                    $shopifyGiftRecipientCustomerId."), Error: (".$shopifyError.")");
                                    continue;
                                }
                                $gifteeActivationLink = $customerActivationUrl['account_activation_url'].'?redeemgift=true';
                            } else {

                                // Get customer details from recharge by shopify customer ID
                                $rechargeCustomerDetails = $this->rechargeObj->getCustomerDetailsByShopifyCustomerId($shopifyGiftRecipientCustomerId);
                                if (!empty($rechargeCustomerDetails->error) || !empty($rechargeCustomerDetails->errors)) {

                                    $rechargeError  = "Unknown error";
                                    if (!empty($rechargeCustomerDetails->error)) {

                                        $rechargeError = json_encode($rechargeCustomerDetails->error);
                                    } else if (!empty($rechargeCustomerDetails->errors)) {

                                        $rechargeError = json_encode($rechargeCustomerDetails->errors);
                                    } else if (empty($rechargeCustomerDetails->customers)) {

                                        $rechargeError = 'Response from recharge is empty or failed to get the 
                                        customer details from recharge';
                                    }
                                    $returnData = [
                                        'error' => $rechargeError
                                    ];

                                    Log::channel('giftCronLog')->info("Failed to get the customer details from 
                                    the recharge(customerId:".$shopifyGiftRecipientCustomerId."), Error: (".$rechargeError.")");
                                    continue;
                                }

                                if (empty($rechargeCustomerDetails->customers) || !isset($rechargeCustomerDetails->customers)) {

                                    $gifteeActivationLink = $gifteeUrlIfGifteeWithActiveShopifyAccount.'?redeemgift=true';
                                } else {
                                    $rechargeCustomerHashToken = trim($rechargeCustomerDetails->customers[0]->hash);
                                    $gifteeActivationLink = $gifteeUrlIfGifteeWithActiveShopifyAccount.
                                    '?hash='.$rechargeCustomerHashToken.'&redeemgift=true';
                                }
                            }

                        } else {

                            $customerCreateRequest = [];
                            $customerCreateRequest = array(
                                "customer" => array(
                                    "first_name" => $gifteeFirstName,
                                    "last_name" => $gifteeLastName,
                                    "email" => $gifteeEmail,
                                    "send_email_welcome" => false,
                                    "addresses" => array(
                                        $recipientShippingAddress
                                    )
                                ),
                            );

                            // Create giftee Shopify account
                            $shopifyCustomerCreateResponse = $this->customerObj->createShopifyCustomer($customerCreateRequest);

                            if (!empty($shopifyCustomerCreateResponse['error']) || !empty($shopifyCustomerCreateResponse['errors']) ||
                                empty($shopifyCustomerCreateResponse)) {

                                $shopifyError  = "Unknown error";
                                if (!empty($shopifyCustomerCreateResponse['error'])) {

                                    $shopifyError = json_encode($shopifyCustomerCreateResponse['error']);
                                } else if (!empty($shopifyCustomerCreateResponse['errors'])) {

                                    $shopifyError = json_encode($shopifyCustomerCreateResponse['errors']);
                                } else if (empty($shopifyCustomerCreateResponse)) {

                                    $shopifyError = 'Failed to create the giftee account in Shopify';
                                }
                                $returnData = [
                                    'error' => $shopifyError
                                ];

                                Log::channel('giftCronLog')->info("Failed to create the giftee account in Shopify(customerCreateRequest:".
                                json_encode($customerCreateRequest)."), Error: (".$shopifyError.")");
                                continue;
                            }

                            $shopifyGifteeCustomerId = $shopifyCustomerCreateResponse['customer']['id'];
                            // Create giftee Shopify account activation_url
                            $customerActivationUrl = $this->customerObj->createAccountActivationLink($shopifyCustomerCreateResponse['customer']['id']);

                            if (!empty($customerActivationUrl['error']) || !empty($customerActivationUrl['errors']) ||
                                empty($customerActivationUrl)) {

                                $shopifyError  = "Unknown error";
                                if (!empty($customerActivationUrl['error'])) {

                                    $shopifyError = json_encode($customerActivationUrl['error']);
                                } else if (!empty($customerActivationUrl['errors'])) {

                                    $shopifyError = json_encode($customerActivationUrl['errors']);
                                } else if (empty($customerActivationUrl)) {

                                    $shopifyError = 'Failed to create the giftee account activation URL';
                                }
                                $returnData = [
                                    'error' => $shopifyError
                                ];

                                Log::channel('giftCronLog')->info("Failed to create the giftee account activation URL(customerId:".
                                $shopifyCustomerCreateResponse['customer']['id']."), Error: (".$shopifyError.")");
                                continue;
                            }

                            $gifteeActivationLink = $customerActivationUrl['account_activation_url'].'?redeemgift=true';
                        }

                        $gifteeIterableEventRequest = [];
                        $gifteeIterableEventRequest = array(
                            "events" => array(
                            array("email" => strtolower($gifteeEmail),
                            "eventName" => "giftee",
                            "dataFields" => array(
                                "giftee_first_name" => $gifteeFirstName,
                                "giftee_last_name" => $gifteeLastName,
                                "gift_note" => $giftNote,
                                "gift_purchaser_name" => $purchaserName,
                                "gift_pack_name" => $giftPackName,
                                "gift_number_charges_until_expiration" => $giftNumberChargesUntilExpiration,
                                "token" => $token,
                                "giftee_activation_link" => $gifteeActivationLink
                                )
                              )
                           )
                        );

                        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($gifteeIterableEventRequest));

                        if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

                            Log::channel('giftCronLog')->info("Giftee iterable event: - Successfully created the
                            event for Email(".strtolower($gifteeEmail).")");

                            $gifteeEventProcessedTemp = array(
                                "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                                "gifteeEmail" => $gifteeEmail,
                                "gifteeShopifyCustomerId" => $shopifyGifteeCustomerId,
                                "purchaserShopifyCustomerId" => $shopifyGiftPurchaserCustomerId,
                                "Status" => "Success"
                            );

                            $this->gifteeProcessedEvents[] = $gifteeEventProcessedTemp;
                            $gifteeEventProcessedTemp = [];
                            $updatedShopifyTags = '';
                            $shopifyTagsToUpdate = [];
                            $shopifyTagsToUpdate = $existingOrderTags;
                            $shopifyTagsToUpdate[] = 'giftee-notified';
                            $updatedShopifyTags = implode(",", $shopifyTagsToUpdate);

                            $orderUpdateRequest = [];
                            $orderUpdateRequest = array (
                                "order" => array (
                                    "id" => $shopifyOrderId,
                                    "tags" => $updatedShopifyTags
                                )
                            );

                            // Update order tags to Shopify
                            $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $shopifyOrderId);
                            if (!empty($shopifyOrderUpdateResponse['error']) || !empty($shopifyOrderUpdateResponse['errors']) ||
                                empty($shopifyOrderUpdateResponse)) {

                                $shopifyError  = "Unknown error";
                                if (!empty($shopifyOrderUpdateResponse['error'])) {

                                    $shopifyError = json_encode($shopifyOrderUpdateResponse['error']);
                                } else if (!empty($shopifyOrderUpdateResponse->errors)) {

                                    $shopifyError = json_encode($shopifyOrderUpdateResponse['errors']);
                                } else if (empty($shopifyOrderUpdateResponse)) {

                                    $shopifyError = 'Failed to update the Shopify order';
                                }
                                $returnData = [
                                    'error' => $shopifyError
                                ];

                                Log::channel('giftCronLog')->info("Failed to update the Shopify order(orderUpdateRequest:".
                                json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
                            }

                        } else {
                            Log::channel('giftCronLog')->info("Giftee iterable event: - Failed to create the event 
                            for Email (".strtolower($gifteeEmail).")");

                            $gifteeEventProcessedTemp = array(
                                "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                                "gifteeEmail" => $gifteeEmail,
                                "gifteeShopifyCustomerId" => $shopifyGifteeCustomerId,
                                "purchaserShopifyCustomerId" => $shopifyGiftPurchaserCustomerId,
                                "Status" => "Failed"
                            );

                            $this->gifteeProcessedEvents[] = $gifteeEventProcessedTemp;
                            $gifteeEventProcessedTemp = [];
                        }
                    }
                }
                $orderCount = count($orders['body']);
            }
            if (empty($orders['link'])) {
                break;
            }
        } while($orderCount == $limit);

        Log::channel('giftCronLog')->info("Completed Giftee event creation cron working");
    }

    /**
     * Gift expiration event creation in the cron job
     */
    public function giftExpirationEventCreation() {

        $shOrders = [];
        $gifteeExpirationEventProcessedTemp = [];
        $currentDate = date('Y-m-d');
        //$currentDate = '2021-09-01';
        $limit  = 200;
        Log::channel('giftCronLog')->info("Started Gift subscription expiration event creation cron working");
        do {
            try {
                if (!empty($shOrders['link'])) {

                    $shOrders = $this->orderObj->getShopifyOrdersByDate($limit, $orders['link'], $currentDate);

                } else {

                    $shOrders = $this->orderObj->getShopifyOrdersByDate(
                        $limit, NULL, $currentDate);
                    
                    if (empty($shOrders['body'])) {
                        Log::channel('giftCronLog')->info("No orders were found in the ".$currentDate." date");
                    }
                }
            }
            catch (Exception $e){

                Log::channel('giftCronLog')->info("An error occurred while fetching the
                order details from Shopify by date, Error:".json_encode($e->getMessage()));
                die();
            }

            //print_r($shOrders); die;
            if (!empty($shOrders) && isset($shOrders['body'])) {

                foreach ($shOrders['body'] as $shOrder) {

                    if (!empty($shOrder['id'])) {

                        $shopifyOrderId = '';
                        $shopifyOrderId = trim($shOrder['id']);
                        $orderLineItemProperties = !empty($shOrder['line_items'][0]['properties']) ? 
                        $shOrder['line_items'][0]['properties'] : '';

                        if (empty($orderLineItemProperties)) {

                            Log::channel('giftCronLog')->info("Skipped: The order LIP is empty
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        $orderTags = $shOrder['tags'];
                        $existingOrderTags = explode(",", $orderTags);
                        $existingOrderTags = array_map('trim', $existingOrderTags);

                        if (in_array('gift-subscription-expiration-notified', $existingOrderTags)) {
                            
                            Log::channel('giftCronLog')->info("Skipped: Already created the gift_subscription_expiring 
                            event for the orderId:".$shOrder['id']);
                            continue;
                        }
                        
                        $subscriptionId = '';
                        $gifteeFirstName = '';
                        $gifteeLastName = '';
                        $gifteeEmail = '';
                        $numberOfCharges = '';
                        $shopifyOrderCustomerId = '';

                        foreach ($shOrder['line_items'][0]['properties'] as $lipKey => $lipValue) {

                            switch ($lipValue['name']) {

                                case (trim($lipValue['name']) == 'giftee_first_name'):
                                    $gifteeFirstName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_last_name'):
                                    $gifteeLastName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_email'):
                                    $gifteeEmail = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'gift_number_charges_until_expiration'):
                                    $numberOfCharges = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
                            }
                        }

                        $shopifyOrderCustomerId = $shOrder['customer']['id'];

                        if (empty($gifteeEmail) || empty($numberOfCharges)) {

                            Log::channel('giftCronLog')->info("Skipped: The order is not a gift order
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }
                        
                        // Get charge Shopify order charge details from recharge by Shopify order ID
                        $orderChargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($shopifyOrderId);
                        if (!empty($orderChargeDetails->error) || !empty($orderChargeDetails->errors) ||
                            empty($orderChargeDetails) || empty($orderChargeDetails->charges)) {

                            $shRechargeError  = "Unknown error";
                            if (!empty($orderChargeDetails->error)) {

                                $shRechargeError = json_encode($orderChargeDetails->error);
                            } else if (!empty($orderChargeDetails->errors)) {

                                $shRechargeError = json_encode($orderChargeDetails->errors);
                            } else if (empty($orderChargeDetails) || empty($orderChargeDetails->charges)) {

                                $shRechargeError = 'Failed to get the charge details by using the given orderId: '.$shopifyOrderId;
                            }
                            $returnData = [
                                'error' => $shRechargeError
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the charge details from 
                            the recharge(OrderId:".$shopifyOrderId."), Error: (".$shRechargeError.")");
                            continue;
                        }

                        $rechargeSubscriptionId = !empty($orderChargeDetails->charges[0]->line_items[0]->subscription_id) ? 
                        trim($orderChargeDetails->charges[0]->line_items[0]->subscription_id) : '';

                        
                        // Get giftee Shopify account details
                        $customerDetails = $this->customerObj->getCustomerDetailBySearch($gifteeEmail);

                        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                            $error = "Unknown error";
                            if (!empty($customerDetails['error'])) {

                                $error = json_encode($customerDetails['error']);
                            } elseif (!empty($customerDetails['errors'])) {
                                
                                $error = json_encode($customerDetails['errors']);
                            }
                            if (empty($error)) {

                            $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $gifteeEmail . ")";
                            }
                            $returnData = [
                                "error" => $error
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the customer details from Shopify(emailAddress:".
                            $gifteeEmail."), Error: (".$error.")");
                            continue;
                        }
                        
                        $gifteeShopifyCustomerId = '';
                        $existingGiftRecipientDetails = '';
                        if (!empty($customerDetails['customers']) || !empty($customerDetails) || isset($customerDetails['customers'])) {

                            foreach ($customerDetails['customers'] as $customerKey => $customerValue) {

                                if (strtolower(trim($gifteeEmail)) == strtolower(trim($customerValue['email']))) {

                                    $gifteeShopifyCustomerId = trim($customerValue['id']);
                                    break;
                                }
                            }
                        }

                        // Get all charge details from recharge by subscription ID
                        $allChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($rechargeSubscriptionId);
                        if (!empty($allChargeDetails->error) || !empty($allChargeDetails->errors) ||
                            empty($allChargeDetails) || empty($allChargeDetails->charges)) {

                            $rechargeError  = "Unknown error";
                            if (!empty($allChargeDetails->error)) {

                                $rechargeError = json_encode($allChargeDetails->error);
                            } else if (!empty($allChargeDetails->errors)) {

                                $rechargeError = json_encode($allChargeDetails->errors);
                            } else if (empty($allChargeDetails) || empty($allChargeDetails->charges)) {

                                $rechargeError = 'Failed to get the charge details by using the given subscriptionId: '.$rechargeSubscriptionId;
                            }
                            $returnData = [
                                'error' => $rechargeError
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the charge details from 
                            the recharge(subscriptionId:".$rechargeSubscriptionId."), Error: (".$rechargeError.")");
                            continue;
                        }

                        $successChargeCount = 0;
                        foreach ($allChargeDetails->charges as $chargeKey => $chargeValue) {

                            if (strtolower(trim($chargeValue->status)) == 'success') {

                                $successChargeCount++;
                            }
                        }

                        $numberOfCharges = (int)$numberOfCharges;
                        $successChargeCount = (int)$successChargeCount;

                        if (($numberOfCharges - 1) == $successChargeCount) {

                            $gifExpirationEventRequest = [];
                            $gifExpirationEventRequest = array(
                                "events" => array(
                                    array("email" => strtolower($gifteeEmail),
                                    "eventName" => "gift_subscription_expiring",
                                    "dataFields" => array(
                                        "subscription_id" => $rechargeSubscriptionId
                                        )
                                    )
                                )
                            );

                            $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($gifExpirationEventRequest));

                            if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

                                Log::channel('giftCronLog')->info("Gift expiration iterable event: - Successfully created the
                                event for Email(".strtolower($gifteeEmail).")");

                                $gifteeExpirationEventProcessedTemp = array(
                                    "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                                    "gifteeEmail" => $gifteeEmail,
                                    "gifteeShopifyCustomerId" => $gifteeShopifyCustomerId,
                                    "purchaserShopifyCustomerId" => $shopifyOrderCustomerId,
                                    "Status" => "Success"
                                );

                                $this->giftExpirationProcessedEvents[] = $gifteeExpirationEventProcessedTemp;
                                $gifteeExpirationEventProcessedTemp = [];

                                $updatedShopifyTags = '';
                                $shopifyTagsToUpdate = [];
                                $shopifyTagsToUpdate = $existingOrderTags;
                                $shopifyTagsToUpdate[] = 'gift-subscription-expiration-notified';
                                $updatedShopifyTags = implode(",", $shopifyTagsToUpdate);

                                $orderUpdateRequest = [];
                                $orderUpdateRequest = array (
                                    "order" => array (
                                        "id" => $shopifyOrderId,
                                        "tags" => $updatedShopifyTags
                                    )
                                );

                                // Update order tags to Shopify
                                $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $shopifyOrderId);
                                if (!empty($shopifyOrderUpdateResponse['error']) || !empty($shopifyOrderUpdateResponse['errors']) ||
                                    empty($shopifyOrderUpdateResponse)) {

                                    $shopifyError  = "Unknown error";
                                    if (!empty($shopifyOrderUpdateResponse['error'])) {

                                        $shopifyError = json_encode($shopifyOrderUpdateResponse['error']);
                                    } else if (!empty($shopifyOrderUpdateResponse->errors)) {

                                        $shopifyError = json_encode($shopifyOrderUpdateResponse['errors']);
                                    } else if (empty($shopifyOrderUpdateResponse)) {

                                        $shopifyError = 'Failed to update the Shopify order';
                                    }
                                    $returnData = [
                                        'error' => $shopifyError
                                    ];

                                    Log::channel('giftCronLog')->info("Failed to update the Shopify order(orderUpdateRequest:".
                                    json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
                                }

                            } else {
                                Log::channel('giftCronLog')->info("Gift expiration iterable event: - Failed to create the event 
                                for Email (".strtolower($gifteeEmail).")");

                                $gifteeExpirationEventProcessedTemp = array(
                                    "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                                    "gifteeEmail" => $gifteeEmail,
                                    "gifteeShopifyCustomerId" => $gifteeShopifyCustomerId,
                                    "purchaserShopifyCustomerId" => $shopifyOrderCustomerId,
                                    "Status" => "Success"
                                );

                                $this->giftExpirationProcessedEvents[] = $gifteeExpirationEventProcessedTemp;
                                $gifteeExpirationEventProcessedTemp = [];
                            }
                        }
                    }
                }
                $shOrderCount = count($shOrders['body']);
            }
            if (empty($shOrders['link'])) {
                break;
            }
        } while($shOrderCount == $limit);
        Log::channel('giftCronLog')->info("Completed Gift subscription expiration event creation cron working");

    }

    /**
     * Gift pack cron report email
     */
    public function sendReportForGiftPackEventCreation() {
        
        $iterableMailRequest = '';
        $reportSendingTime =  new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');        
        $reportSendingTime = $reportSendingTime->toDayDateTimeString();
        $gifteeEventCreatedCount = 0;
        $giftExpirationEventCreatedCount = 0;
        $gifteeEventCreatedCount = count($this->gifteeProcessedEvents);
        $giftExpirationEventCreatedCount = count($this->giftExpirationProcessedEvents);

        if ($gifteeEventCreatedCount > 30 || $giftExpirationEventCreatedCount > 30) {

            $this->setPaths();
            $this->getLogFilesList();
            $this->setWithLogFiles();
        }
        
        $commonContent = '';
        $recordDetails = '';
        $recordDetailsGifteeTemp = '';
        $recordDetailsExpirationTemp = '';
        if ($gifteeEventCreatedCount > 0 && $gifteeEventCreatedCount <= 30) {
            
            if (!empty($this->gifteeProcessedEvents)) {           
                $recordDetailsGifteeTemp .= "
                <table>
                    <tr>
                        <td>#</td>                
                        <td>GifteeName</td>
                        <td>GifteeEmail</td> 
                        <td>GifteeShopifyCustomerId</td> 
                        <td>PurchaserShopifyCustomerId</td>
                        <td>Status</td>                       
                    </tr>";

                $i = 1;
                foreach ($this->gifteeProcessedEvents as $record) {
                            
                    $recordDetailsGifteeTemp .= "
                    <tr>
                        <td>".$i."</td>                             
                        <td>".$record['gifteeName']."</td> 
                        <td>".$record['gifteeEmail']."</td>  
                        <td>".$record['gifteeShopifyCustomerId']."</td>
                        <td>".$record['purchaserShopifyCustomerId']."</td>
                        <td>".$record['Status']."</td>
                    </tr>";
                    $i++;            
                }
                $recordDetailsGifteeTemp .= '</table>';
            }
        }
        if ($giftExpirationEventCreatedCount > 0 && $giftExpirationEventCreatedCount <= 30) {
            
            if (!empty($this->giftExpirationProcessedEvents)) {           
                $recordDetailsExpirationTemp .= "
                <table>
                    <tr>
                        <td>#</td>                
                        <td>GifteeName</td>
                        <td>GifteeEmail</td> 
                        <td>GifteeShopifyCustomerId</td> 
                        <td>PurchaserShopifyCustomerId</td>
                        <td>Status</td>                       
                    </tr>";

                $i = 1;
                foreach ($this->giftExpirationProcessedEvents as $record) {
                            
                    $recordDetailsExpirationTemp .= "
                    <tr>
                        <td>".$i."</td>                             
                        <td>".$record['gifteeName']."</td> 
                        <td>".$record['gifteeEmail']."</td>  
                        <td>".$record['gifteeShopifyCustomerId']."</td>
                        <td>".$record['purchaserShopifyCustomerId']."</td>
                        <td>".$record['Status']."</td>
                    </tr>";
                    $i++;            
                }
                $recordDetailsExpirationTemp .= '</table>';
            }
        }

        $recipientEmails = 'testing@p80w.com';
        // $recipientEmails = 'mikegordon79@gmail.com';

        // Processing gift event created report
        $subject = "Nurish-Dev Shopify gift pack event creation notification - ".$reportSendingTime;
        
        if ($gifteeEventCreatedCount == 0 && $giftExpirationEventCreatedCount == 0) {

            Log::channel('giftCronLog')->info("No giftee and gift_subscription_expiring events were created in 
            iterable during this execution");
            $commonContent = '<p>No giftee and gift_subscription_expiring events were created in iterable 
            during this execution</p>';

        } elseif ($gifteeEventCreatedCount > 0 && $gifteeEventCreatedCount <= 30 && 
        $giftExpirationEventCreatedCount == 0) {

            $commonContent = '<p>Please see the below details for the gift events that were created during this 
            execution.</p>';

            $recordDetails .= '<b>Giftee events:</b>';
            $recordDetails .= $recordDetailsGifteeTemp;
            $recordDetails .= '<br>';
            $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
            $recordDetails .= '<br>No gift_subscription_expiring events were created in iterable during this execution';

        } elseif ($giftExpirationEventCreatedCount > 0 && $giftExpirationEventCreatedCount <= 30 && 
        $gifteeEventCreatedCount == 0) {

            $commonContent = '<p>Please see the below details for the gift events that were created during this 
            execution.</p>';

            $recordDetails .= '<b>Giftee events:</b>';
            $recordDetails .= '<br>No giftee events were created in iterable during this execution';
            $recordDetails .= '<br><br>';
            $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
            $recordDetails .= $recordDetailsExpirationTemp;

        } elseif ($gifteeEventCreatedCount > 30 && $giftExpirationEventCreatedCount > 30) {

            $gifteeFileUrl = url('/').'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
            $this->gifteeEventsCreatedReport;
            $gifteeExpirationFileUrl = url('/').'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
            $this->giftExpirationEventsCreatedReport;
            $commonContent = '<p>Please see the below details for the gift events that were created during this 
            execution.</p>';
            
            $recordDetails .= '<b>Giftee events:</b>';
            $recordDetails = 'Please <a href="'.$gifteeFileUrl.'">click here</a> to download the file.';
            $recordDetails .= '<br><br>';
            $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
            $recordDetails .= 'Please <a href="'.$gifteeExpirationFileUrl.'">click here</a> to download the file.';

        } elseif ($gifteeEventCreatedCount > 0 && $gifteeEventCreatedCount <= 30 && 
        $giftExpirationEventCreatedCount > 0 && $giftExpirationEventCreatedCount <= 30) {

            $commonContent = '<p>Please see the below details for the gift events that were created during this 
            execution.</p>';

            $recordDetails .= '<b>Giftee events:</b>';
            $recordDetails .= $recordDetailsGifteeTemp;
            $recordDetails .= '<br>';
            $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
            $recordDetails .= $recordDetailsExpirationTemp;

        } elseif ($gifteeEventCreatedCount == 0 && $giftExpirationEventCreatedCount > 30) {

            $gifteeExpirationFileUrl = url('/').'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
            $this->giftExpirationEventsCreatedReport;
            $commonContent = '<p>Please see the below details for the gift events that were created during this 
            execution.</p>';

            $recordDetails .= '<b>Giftee events:</b>';
            $recordDetails .= '<br>No giftee events were created in iterable during this execution';
            $recordDetails .= '<br><br>';
            $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
            $recordDetails .= 'Please <a href="'.$gifteeExpirationFileUrl.'">click here</a> to download the file.';
            
        } elseif ($giftExpirationEventCreatedCount == 0 && $gifteeEventCreatedCount > 30) {

            $gifteeFileUrl = url('/').'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
            $this->gifteeEventsCreatedReport;
            $commonContent = '<p>Please see the below details for the gift events that were created during this 
            execution.</p>';

            $recordDetails .= '<b>Giftee events:</b>';
            $recordDetails .= 'Please <a href="'.$gifteeFileUrl.'">click here</a> to download the file.';
            $recordDetails .= '<br><br>';
            $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
            $recordDetails .= '<br>No gift_subscription_expiring events were created in iterable during this execution';

        } elseif ($gifteeEventCreatedCount > 0 && $gifteeEventCreatedCount <= 30 && 
        $giftExpirationEventCreatedCount > 30) {

            $gifteeExpirationFileUrl = url('/').'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
            $this->giftExpirationEventsCreatedReport;
            $commonContent = '<p>Please see the below details for the gift events that were created during this 
            execution.</p>';

            $recordDetails .= '<b>Giftee events:</b>';
            $recordDetails .= $recordDetailsGifteeTemp;
            $recordDetails .= '<br>';
            $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
            $recordDetails .= 'Please <a href="'.$gifteeExpirationFileUrl.'">click here</a> to download the file.';

        } elseif ($giftExpirationEventCreatedCount > 0 && $giftExpirationEventCreatedCount <= 30 && 
        $gifteeEventCreatedCount > 30) {

            $gifteeFileUrl = url('/').'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
            $this->gifteeEventsCreatedReport;
            $commonContent = '<p>Please see the below details for the gift events that were created during this 
            execution.</p>';

            $recordDetails .= '<b>Giftee events:</b>';
            $recordDetails .= 'Please <a href="'.$gifteeFileUrl.'">click here</a> to download the file.';
            $recordDetails .= '<br><br>';
            $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
            $recordDetails .= $recordDetailsExpirationTemp;

        }
        $iterableMailRequest = array(
            "campaignId" => intval(env('GIFT_PACK_EVENT_CRON_CAMPAIGN_ID', true)),
            "recipientEmail" => $recipientEmails,
            "dataFields" => array(                
                "recordDetails" => $recordDetails,
                "subject" => $subject,
                "commonContent" => $commonContent
            ),
        );
        
        if(!empty($iterableMailRequest)) {
            
            $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

            if (strtolower(trim($mailTriggerResponse->code)) == 'success') {           
                
                Log::channel('giftCronLog')->info("giftee event created report mail sent");
    
            } else {
                
                Log::channel('giftCronLog')->info('giftee event created report mail not sent');
                Log::channel('giftCronLog')->info(json_encode($mailTriggerResponse));
                $error = !empty($mailTriggerResponse->code) ? $mailTriggerResponse->code : '';
    
                if (empty($error)) {
    
                    $returnData = [
                        'error' => 'Failed to send mail(' . $recipientEmails . ')',
                    ];
                } else {
    
                    $returnData = [
                        'error' => $error . ' (' . $recipientEmails . ')',
                    ];
                }
                Log::channel('giftCronLog')->info($returnData['error']);
            }
        }
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../public/GiftPackEventCreatedReport/";
    }

    public function getLogFilesList() {
        
        $this->reportFileDirectory = date('dmY');

        $this->logPath = $this->logDirectory . $this->reportFileDirectory ;
        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);
        }
        
        $this->gifteeEventsCreatedReport = "giftee-event-processed-list-".date("Y_m_d_H_i").".csv";
        $this->giftExpirationEventsCreatedReport = "gift-expiration-event-processed-list-".date("Y_m_d_H_i").".csv";

        $this->logFiles = array(
            'gifteeEventProcessed' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->gifteeEventsCreatedReport,
                "mode"     => "",
                "columns"  => array(
                    "GifteeName",
                    "GifteeEmail",
                    "GifteeShopifyCustomerId",
                    "PurchaserShopifyCustomerId",                   
                    "Status"
                )
            ),
            'giftExpireEventProcessed' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->giftExpirationEventsCreatedReport,
                "mode"     => "",
                "columns"  => array(
                    "GifteeName",
                    "GifteeEmail",
                    "GifteeShopifyCustomerId",
                    "PurchaserShopifyCustomerId",                          
                    "Status"
                )
            )        
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {

            foreach ($this->logFiles as $logType=>$fileDetails){

                if ($fileDetails['type']=="csv"){
                    $mode = !empty($fileDetails['mode']) ? $fileDetails['mode'] : "w";

                    if ($mode != "r"){
                        $fileName    = $fileDetails['filename'];
                        $filePointer = fopen($fileName,$mode);
                        $columns     = $fileDetails['columns'];
                        fputcsv($filePointer,$columns);
                        chmod($fileName, 0777);
                        fclose($filePointer);
                    }
                }
            }
        }
    }

    /**
     * Create random password
     */
    // public function createRandomPassword() {

    //     $randomPassword = '';
    //     $alphabet = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
    //     $password = array();
    //     $alphaLength = strlen($alphabet) - 1;
    //     for ($i = 0; $i < 16; $i++) {
    //         $n = rand(0, $alphaLength);
    //         $password[] = $alphabet[$n];
    //     }
    //     $randomPassword = implode($password);
    //     return $randomPassword;
    // }

    /**
     * Response function
     */
    public function returnData($status, $message, $error, $jsonCallback = null) {

        $returnData = array(
            'status' => $status,
            'message' => $message,
            'body' => $error

        );
        if (!empty($jsonCallback)) {

            echo $jsonCallback . '(' . json_encode($returnData) . ')';
            die();
        } else {

            return response()->json($returnData);
        }
    }

}
