<?php

namespace App\Jobs;

require_once(dirname(__FILE__).'/../../vendor/autoload.php');

use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Support\Facades\File;
use Config;
use Log;

use App\Model\Shopify\ExtentedShopifyAPI;
use OhMyBrew\BasicShopifyAPI;

use App\Model\IterableApi;
use DB;


class CreateShopifyCustomerJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    public $email;
    public $userName;
    public $partnerValue;
    public $session_id;
    public $shopify_api_version;
    public $iterableObj;
    
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($customerDetails)
    {
        //
        $this->email = $customerDetails['email'];
        $this->userName = $customerDetails['userName'];
        $this->partnerValue = (isset($customerDetails['partnerValue']))?$customerDetails['partnerValue'] : '';
        $this->session_id = (isset($customerDetails['sessionId']))?$customerDetails['sessionId'] : '';
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    /**
     * Execute the job.
     * @return void
     */
    public function handle()
    {

        //$api = new BasicShopifyAPI();
        $api = new ExtentedShopifyAPI();
        
        $this->iterableObj = new IterableApi();

        $password   = Config::get('shopify.passwordQuiz');
        $shopDomain = Config::get('shopify.shopDomainQuiz');

        $api->setShop($shopDomain);
        $api->setAccessToken($password);

        $randomPassword = $this->generateRandomString(8);
        
        $tags = [];
        if(trim($this->partnerValue) != ''){
            $tags[] = "partner:".$this->partnerValue;
        }
      
        $params = array(
            'customer' => array(
                'email' => $this->email,
                'first_name' => $this->userName,
                "send_email_welcome" => "false",
                "password" => $randomPassword,
                "password_confirmation" => $randomPassword,
                "tags" => $tags,
                "accepts_marketing" => "true"
            )
        );
        //sleep(1);
        Log::info($this->partnerValue);

        Log::info(json_encode($params));
        $retry = false;
        $try = 1;
        do{
            Log::info($try.' try-create shopify customer ');
            $result = $api->rest('POST', '/admin/api/'.$this->shopify_api_version.'/customers.json', $params); 
            if(!isset($result->body->customer->id)){
                $retry = true;
                $try++;
            }
            if(isset($result->body->customer->id)){
                $this->updateRegistrationAuthInIterable();
            }
            
        } while($retry && $try <=3);
        Log::info(json_encode($result));

        if(empty($result->errors))  //success
        {
            // TO DO
        }
        else  //error
        {
            // TO DO
        }

    }
    
    public function updateRegistrationAuthInIterable(){

         if(!empty($this->email)){

            $registrationAuth = DB::table('users')->select('registration_auth')->where('user_email', '=', $this->email)->get();


            if(isset($registrationAuth[0]->registration_auth) && !empty($registrationAuth[0]->registration_auth)){
                $userIterableUpdateRequest = array(
                    "email" => $this->email,
                    "dataFields" => array(
                        "registrationAuth" => $registrationAuth[0]->registration_auth,
                        "sessionId" => $this->session_id
                    ),
                );
                $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));
            }

        }

    }
    
    function generateRandomString($length) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }
}
