<?php

namespace App\Jobs;

use Log;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Config;
use App\Model\Shopify\ExtentedShopifyAPI;
use OhMyBrew\BasicShopifyAPI;
use DB;
use App\Model\IterableApi;
use App\Model\RechargeApi;

class OrderTaggingJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;


    public $webhook;
    public $shopify_api_version;
    public $rechargeObj;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($webhook)
    {
        $this->webhook = $webhook;
        $this->shopify_api_version = env('SHOPIFY_API_VERSION', true);
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        //$api = new BasicShopifyAPI();
        $api = new ExtentedShopifyAPI();
        $this->rechargeObj = new RechargeApi();

        $password   = Config::get('shopify.passwordOrderTag');
        $shopDomain = Config::get('shopify.shopDomainOrderTag');

        $api->setShop($shopDomain);
        $api->setAccessToken($password);

        $webhookData = json_decode($this->webhook);
        // $webhookData = json_decode($webhookData);
        $lineItems    = $webhookData->line_items;
        $orderId      = $webhookData->id;
        $orderTags    = $webhookData->tags;
        $orderEmail   = $webhookData->email;
        $customerId   = $webhookData->customer->id;
        $customerTags = $webhookData->customer->tags;
        
        $newOrderTag = [];
        $newCustomerTag = [];
        $oTagFlag = 0;
        $giftFirstOrderStatus = 0;
        $giftNotificationDate = '';

        if(!empty($lineItems)){

            foreach($lineItems as $lineitem){
                
                $lineItemProperties = $lineitem->properties;
                if(!empty($lineItemProperties)){

                    foreach($lineItemProperties as $lineItemProperty){

                        $name  = trim($lineItemProperty->name);
                        $value = trim($lineItemProperty->value);
                        $value = str_replace(',', '-', $value);
                        
                        $oTag  = '';
                        

                        // Set Special Pack tag
                        if($name == 'special_pack_type'){  
                            
                            $newOrderTag[]    = 'special pack:'.$value;
                            $newCustomerTag[] = 'special pack:'.$value;
                        }

                        // Set Free Override Shipping tag
                        if($name == 'override_free_shipping' && $value == 'true'){
                            
                            $newOrderTag[] = 'free-shipping-override';
                            
                        }

                        // Set Partner tag
                        if($name == 'partner'){
                            
                            $oTag = 'partner:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'partner:'.$value;
                            
                        }
                        // Set campaign tag
                        if($name == 'campaign'){
                            
                            $oTag = 'campaign:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'campaign:'.$value;
                            
                        }
                        // Set source tag
                        if($name == 'source'){
                           
                            $oTag = 'source:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'source:'.$value;
                           
                        }
                        // Set medium tag
                        if($name == 'medium'){
                           
                            $oTag = 'medium:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'medium:'.$value;
                           
                        }
                        // Set content tag
                        if($name == 'content'){
                           
                            $oTag = 'content:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'content:'.$value;
                           
                        }
                        // Set term tag
                        if($name == 'term'){
                           
                            $oTag = 'term:'.$value;
                            if(strlen($oTag) > 40){
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            
                            $newOrderTag[]    =  $oTag;
                            $newCustomerTag[] = 'term:'.$value;
                           
                        }

                        // Set gift pack order tags
                        if ($name == 'giftee_email') {

                            $oTag = 'gift';
                            if (strlen($oTag) > 40) {
                               $oTag = substr($oTag,0,40); 
                               $oTagFlag++;
                            }
                            $newOrderTag[]    =  $oTag;

                            // Get charge details from recharge by shopify order ID
                            $chargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($orderId);
                            if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
                                empty($chargeDetails) || empty($chargeDetails->charges)) {

                                $rechargeError  = "Unknown error";
                                if (!empty($chargeDetails->error)) {

                                    $rechargeError = json_encode($chargeDetails->error);
                                } else if (!empty($chargeDetails->errors)) {

                                    $rechargeError = json_encode($chargeDetails->errors);
                                } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                                    $rechargeError = 'Failed to get the charge details by using the given orderId: '.$orderId;
                                }
                                $returnData = [
                                    'error' => $rechargeError
                                ];

                                Log::channel('orderTag')->info("Failed to get the charge details from 
                                the recharge(OrderId:".$orderId."), Error: (".$rechargeError.")");
                                //return $this->returnData(600, "error", $returnData);
                            }
                            $subscriptionAddressId = $chargeDetails->charges[0]->address_id;

                            // Get order by subscription addressId
                            if (!empty($subscriptionAddressId)) {
                                $rechargeOrders = $this->rechargeObj->getRechargeOrdersByAddressId($subscriptionAddressId);
                                if (!empty($rechargeOrders->error) || !empty($rechargeOrders->errors) ||
                                    empty($rechargeOrders) || empty($rechargeOrders->orders)) {

                                    $rechargeError  = "Unknown error";
                                    if (!empty($rechargeOrders->error)) {

                                        $rechargeError = json_encode($rechargeOrders->error);
                                    } else if (!empty($rechargeOrders->errors)) {

                                        $rechargeError = json_encode($rechargeOrders->errors);
                                    } else if (empty($rechargeOrders) || empty($rechargeOrders->charges)) {

                                        $rechargeError = 'Failed to get the recharge orders by using the addressId: '.$subscriptionAddressId;
                                    }
                                    $returnData = [
                                        'error' => $rechargeError
                                    ];

                                    Log::channel('orderTag')->info("Failed to get the order details from 
                                    the recharge(OrderId:".$orderId.",addressId:".$subscriptionAddressId."), Error: (".$rechargeError.")");
                                    //return $this->returnData(600, "error", $returnData);
                                }
                            } else {
                                Log::channel('orderTag')->info("The addressId in the charge is empty(OrderId:".$orderId.")");
                            }

                            $rechargeOrderCount = !empty($rechargeOrders->orders) ? count($rechargeOrders->orders) : '';
                            if (!empty($rechargeOrderCount) && $rechargeOrderCount > 0) {
                                $giftFirstOrderStatus = 1;
                                $newOrderTag[]    =  'unclaimed';
                            }
                        }

                        // Set gift notification date order tags
                        if ($name == 'gift_notification_date') {
                            $giftNotificationDate = trim($lineItemProperty->value);
                        }
                    }
                }
            }
            
            if ($giftFirstOrderStatus == 1 && !empty($giftNotificationDate)) {
                $newOrderTag[]    =  'giftNotificationDate:'.$giftNotificationDate;
            }

            if($oTagFlag > 0){
                $newOrderTag[]    =  'utm_trimmed';
            }
            
            $newOrderTag    = implode(',',$newOrderTag);
            $newCustomerTag = implode(',',$newCustomerTag);

            
            $newTag = $orderTags.", ".$newOrderTag;
            $params = array(
                'order' => array(
                    'id' => $orderId,
                    'tags' => $newTag
                )
            );
            
            $method = 'PUT';
            $url = '/admin/api/'.$this->shopify_api_version.'/orders/'.$orderId.'.json';
           
            $retry = false;
            $try = 1;
            do{
                $result = $api->rest($method, $url, $params);
                if(!isset($result->body->order)){
                    $retry = true;
                    $try++;
                }
            } while($retry && $try <=3);

            if(empty($result->errors) && !empty($result->body->order)){ // success

                Log::channel('orderTag')->info("Successfuly updated the order tags - ".$orderId);
                Log::channel('orderTag')->info(
                    "Order Id = ".$orderId.
                    " => OrderTag => ".$orderTags.
                    " => New tags => ". $newTag
                );
                if(!empty($newCustomerTag)){

                    $this->seTCustomerTag($api, $customerId, $customerTags, $newCustomerTag, $orderId);
                    $this->setIterableTag($orderEmail, $customerTags, $newCustomerTag);
		        }
               
            }
            else{
                Log::channel('orderTag')->info("Failed to update the order tags - ".$orderId);
                Log::channel('orderTag')->info(json_encode($result));
            }

        }

        
    }
    
    function seTCustomerTag($api, $customerId, $customerTags, $newCustomerTag, $orderId){

        $newTag = $customerTags.", ".$newCustomerTag;
 
        $params = array(
            'customer' => array(
                'id' => $customerId,
                'tags' => $newTag
            )
        );

        $method = 'PUT';
        $url = '/admin/api/'.$this->shopify_api_version.'/customers/'.$customerId.'.json';
        
        $retry = false;
	    $try = 1;
        do{
            $result = $api->rest($method, $url, $params);
            if(!isset($result->body->customer->id)){
                 $retry = true;
                 $try++;
            }
        } while($retry && $try <=3);
        
        Log::channel('orderTag')->info(json_encode($result));
        if(empty($result->errors) && !empty($result->body->customer)){ // success

            Log::channel('orderTag')->info("Successfuly updated the customer tags - ".$customerId." for the order ".$orderId);
            Log::channel('orderTag')->info(
                "Order Id = ".$orderId.
                " => Customer Id => ".$customerId.
                " => Customer tag => ".$customerTags.
                " => New tags => ". $newTag
            );

        }
        else{
            Log::channel('orderTag')->info("Failed to update the Customer tags - ".$customerId." for the order ".$orderId);
             Log::channel('orderTag')->info(json_encode($result));
          }

    }
    function setIterableTag($email, $customerTags, $newCustomerTag){

        $iterableModel = new IterableApi();
        $userData = $iterableModel->getIterableSubscription($email);
 
        Log::channel('orderTag')->info(json_encode($userData));
        $newTag = $customerTags.", ".$newCustomerTag; 
        $iterableRequest = array(
            "email" => $email,
            "dataFields" => array(
                "tags" => $newTag
            )
        );


        $userIterableUpdateResponse = $iterableModel->updateUserDataFields(json_encode($iterableRequest));

        if(strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            Log::channel('orderTag')->info("Successfuly updated the customer tags for the customer - ".$email);
        }
        else{
            Log::channel('orderTag')->info("Failed to update customer tags for the customer - ".$email);
            Log::channel('orderTag')->info(json_encode($userIterableUpdateResponse));
        }
    }
    
}
