<?php

use Monolog\Handler\StreamHandler;
use Monolog\Handler\SyslogUdpHandler;

return [

    /*
    |--------------------------------------------------------------------------
    | Default Log Channel
    |--------------------------------------------------------------------------
    |
    | This option defines the default log channel that gets used when writing
    | messages to the logs. The name specified in this option should match
    | one of the channels defined in the "channels" configuration array.
    |
    */

    'default' => env('LOG_CHANNEL', 'stack'),

    /*
    |--------------------------------------------------------------------------
    | Log Channels
    |--------------------------------------------------------------------------
    |
    | Here you may configure the log channels for your application. Out of
    | the box, Laravel uses the Monolog PHP logging library. This gives
    | you a variety of powerful log handlers / formatters to utilize.
    |
    | Available Drivers: "single", "daily", "slack", "syslog",
    |                    "errorlog", "monolog",
    |                    "custom", "stack"
    |
    */

    'channels' => [
        'stack' => [
            'driver' => 'stack',
            'channels' => ['daily'],
            'ignore_exceptions' => false,
        ],

        'single' => [
            'driver' => 'single',
            'path' => storage_path('logs/laravel.log'),
            'level' => 'debug',
        ],

        'daily' => [
            'driver' => 'daily',
            'path' => storage_path('logs/laravel.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'slack' => [
            'driver' => 'slack',
            'url' => env('LOG_SLACK_WEBHOOK_URL'),
            'username' => 'Laravel Log',
            'emoji' => ':boom:',
            'level' => 'critical',
        ],

       	 'email' => [
	     'driver' => 'daily',
	     'path' => storage_path('logs/email.log'),
	     'level' => 'debug',
	     'days' => 14
         //'permission' => 0664,
	    ],
        
         'cron' => [
             'driver' => 'daily',
             'path' => storage_path('logs/cron.log'),
             'level' => 'debug',
             'days' => 14,
             'permission' => 0664,
        ],
        
        'abandonedCart' => [
            'driver' => 'daily',
            'path' => storage_path('logs/abandonedCart.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'shipmentDeliveryTagging' => [
            'driver' => 'daily',
            'path' => storage_path('logs/shipmentDeliveryTagging.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'subscriptionDiscountCron' => [
            'driver' => 'daily',
            'path' => storage_path('logs/subscriptionDiscountCron.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'subscriptionMetaUpdate' => [
            'driver' => 'daily',
            'path' => storage_path('logs/subscriptionMetaUpdate.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'orderCancelRefundEndpointLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/orderCancelRefundEndpointLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'orderCancelRerouteEndpointLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/orderCancelRerouteEndpointLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'orderExtractedTagStatusCheckEndpointLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/orderExtractedTagStatusCheckEndpointLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'giftPackLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/giftPackLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'giftCronLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/giftCronLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'newsLetterEmailLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/newsLetterEmailLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
         'ohMyBrew' => [
             'driver' => 'daily',
             'path' => storage_path('logs/ohMyBrew.log'),
             'level' => 'debug',
             'days' => 14,
             'permission' => 0664,
        ],
         'orderTag' => [
             'driver' => 'daily',
             'path' => storage_path('logs/orderTagging.log'),
             'level' => 'debug',
             'days' => 14,
             'permission' => 0664,
        ],
         'shipmentWebhookIterable' => [
            'driver' => 'daily',
            'path' => storage_path('logs/shipmentWebhookIterable.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'papertrail' => [
            'driver' => 'monolog',
            'level' => 'debug',
            'handler' => SyslogUdpHandler::class,
            'handler_with' => [
                'host' => env('PAPERTRAIL_URL'),
                'port' => env('PAPERTRAIL_PORT'),
            ],
        ],

        'stderr' => [
            'driver' => 'monolog',
            'handler' => StreamHandler::class,
            'formatter' => env('LOG_STDERR_FORMATTER'),
            'with' => [
                'stream' => 'php://stderr',
            ],
        ],

        'syslog' => [
            'driver' => 'syslog',
            'level' => 'debug',
        ],

        'errorlog' => [
            'driver' => 'errorlog',
            'level' => 'debug',
        ],
    ],

];
